Package: CalibrateSSB
Type: Package
Title: Weighting and Estimation for Panel Data with Non-Response
Version: 1.4.0
Date: 2026-05-21
Authors@R: c(person(given = "Øyvind", family = "Langsrud", role = c("aut", "cre"),
                    email = "oyl@ssb.no",
                    comment = c(ORCID = "0000-0002-1380-4396")))
Depends: R (>= 3.0.0), survey
Imports: methods
Suggests: ReGenesees, testthat (>= 2.1.0)
Description: Functions to calculate weights, estimates of changes and corresponding variance estimates for panel data with non-response. Partially overlapping samples are handled. Initially, weights are calculated by linear calibration. By default, the 'survey' package is used for this purpose. It is also possible to use 'ReGenesees', which can be installed from <https://github.com/DiegoZardetto/ReGenesees>. Variances of linear combinations (changes and averages) and ratios are calculated from a covariance matrix based on residuals according to the calibration model. The methodology was presented at the conference, The Use of R in Official Statistics, and is described in Langsrud (2016) <http://www.revistadestatistica.ro/wp-content/uploads/2016/06/RRS2_2016_A021.pdf>.  
License: GPL-2
Encoding: UTF-8
URL: https://github.com/statisticsnorway/ssb-calibratessb,
        https://statisticsnorway.github.io/ssb-calibratessb/
BugReports: https://github.com/statisticsnorway/ssb-calibratessb/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-21 09:12:26 UTC; oyl
Author: Øyvind Langsrud [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1380-4396>)
Maintainer: Øyvind Langsrud <oyl@ssb.no>
Repository: CRAN
Date/Publication: 2026-05-21 10:22:03 UTC
Built: R 4.6.0; ; 2026-05-22 00:16:33 UTC; unix

Package: CalibrateSSB
Type: Package
Title: Weighting and Estimation for Panel Data with Non-Response
Version: 1.4.0
Date: 2026-05-21
Authors@R: c(person(given = "Øyvind", family = "Langsrud", role = c("aut", "cre"),
                    email = "oyl@ssb.no",
                    comment = c(ORCID = "0000-0002-1380-4396")))
Depends: R (>= 3.0.0), survey
Imports: methods
Suggests: ReGenesees, testthat (>= 2.1.0)
Description: Functions to calculate weights, estimates of changes and corresponding variance estimates for panel data with non-response. Partially overlapping samples are handled. Initially, weights are calculated by linear calibration. By default, the 'survey' package is used for this purpose. It is also possible to use 'ReGenesees', which can be installed from <https://github.com/DiegoZardetto/ReGenesees>. Variances of linear combinations (changes and averages) and ratios are calculated from a covariance matrix based on residuals according to the calibration model. The methodology was presented at the conference, The Use of R in Official Statistics, and is described in Langsrud (2016) <http://www.revistadestatistica.ro/wp-content/uploads/2016/06/RRS2_2016_A021.pdf>.  
License: GPL-2
Encoding: UTF-8
URL: https://github.com/statisticsnorway/ssb-calibratessb,
        https://statisticsnorway.github.io/ssb-calibratessb/
BugReports: https://github.com/statisticsnorway/ssb-calibratessb/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-21 09:12:26 UTC; oyl
Author: Øyvind Langsrud [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1380-4396>)
Maintainer: Øyvind Langsrud <oyl@ssb.no>
Repository: CRAN
Date/Publication: 2026-05-21 10:22:03 UTC

# Generated by roxygen2: do not edit by hand

S3method(print,calSSB)
S3method(print,calSSBwide)
export(AkuData)
export(CalSSBobj)
export(CalSSBobjReGenesees)
export(CalibrateSSB)
export(CalibrateSSBpanel)
export(CrossStrata)
export(LagDiff)
export(LinCombMatrix)
export(MatchVarNames)
export(OrderedVarNames)
export(PanelEstimation)
export(Period)
export(PeriodDiff)
export(WideFromCalibrate)
importFrom(methods,formalArgs)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,lm.influence)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,predict)
importFrom(stats,resid)
importFrom(stats,update)
importFrom(stats,weights)
importFrom(survey,calibrate)
importFrom(survey,svydesign)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,tail)

CalibratePackageReGenesees = function(netSample,calmodel=NULL,popTotals=NULL,y=NULL,by = NULL,partition=NULL,
                          popData=NULL,samplingWeights=NULL,bounds=c(-Inf,Inf),calfun="linear",
                          onlyTotals=FALSE,ids,...){
  #warning("The non-CRAN package, ReGenesees, is needed.")
  # Add this line to NAMESPACE when ReGenesees is on CRAN
  #importFrom(ReGenesees,e.svydesign,pop.template,fill.template,get.residuals,e.calibrate,svystatTM)
  # and add ReGenesees to Depends or Suggests in DESCRIPTION
  
  if (requireNamespace("ReGenesees", quietly = TRUE)) {
    e.svydesign   <- ReGenesees::e.svydesign
    pop.template  <- ReGenesees::pop.template
    fill.template <- ReGenesees::fill.template
    e.calibrate   <- ReGenesees::e.calibrate
    get.residuals <- ReGenesees::get.residuals
    svystatTM     <- ReGenesees::svystatTM
  } else {
    stop("The package ReGenesees, is needed.")
  }
  
  
  desReGenesees <- e.svydesign(netSample,ids=asFormula(ids), weights =asFormula(samplingWeights))
  
  if(is.null(popTotals)){
    if(is.null(partition))
      popTemplate <- pop.template(data=desReGenesees, calmodel=as.formula(calmodel))
    else
      popTemplate <- pop.template(data=desReGenesees, calmodel=as.formula(calmodel),partition=asFormula(partition))
    popTotals   <- fill.template(universe=popData,template= popTemplate)
  } else
    if(!is.null(partition)) warning("Partition as input has no effect when popTotals is specified and ReGenesees is used.")


  if(onlyTotals) return(popTotals)
  calReGenesees <- e.calibrate(design=desReGenesees, df.population=popTotals,bounds= bounds,calfun=calfun)
  w=weights(calReGenesees) #########   BARE NETTO ##################
  
  ###calReGenesees <<- calReGenesees ####################

  resids = get.residuals(calReGenesees,asFormula(y), scale = "no")

  estTM=NULL

  if(!is.null(y)){
    if(is.list(y) | is.list(by)){
      if(is.list(y) & is.list(by)) {if(length(y)!=length(by)) stop("length(y)==length(by) must be TRUE")}
      else{
        if(is.list(y)){
          if(is.null(by))
            by = vector("list",length(y))
          else{
            by_ = by
            by = y
            for(i in 1:length(y)) by[[i]] = by_
          }
        }else{
          y_ = y
          y = by
          for(i in 1:length(y)) y[[i]] = y_
        }

      }
      estTM = y
      for(i in 1:length(y)) estTM[[i]] = svystatTM(calReGenesees,y=asFormula(y[[i]]),by=asFormula(by[[i]]),...)

    } else
    {
      estTM =  svystatTM(calReGenesees,y=asFormula(y),by=asFormula(by),...)
    }

  }
  if(is.null(estTM)) return(list(popTotals=popTotals,w=w))
  list(popTotals=popTotals,w=w,estTM=estTM,resids=resids)
}

#' Weighting and Estimation for Panel Data with Non-Response
#' 
#' CalibrateSSB is an R-package that handles repeated surveys with partially
#' overlapping samples. Initially the samples are weighted by linear
#' calibration using known or estimated population totals. A robust model based
#' covariance matrix for all relevant estimated totals is calculated from the
#' residuals according to the calibration model. Alternatively a design based
#' covariance matrix is calculated in a very similar way. A cluster robust
#' version is also possible. In the case of estimated populations totals the
#' covariance matrix is adjusted by utilizing the theory of Särndal and
#' Lundström (2005). Variances of linear combinations (changes and averages)
#' and ratios are calculated from this covariance matrix. The linear
#' combinations and ratios can involve variables within and/or between sample
#' waves. \cr
#' 
#' @name CalibrateSSB-package
#' 
#' @references Langsrud, Ø (2016): \dQuote{A variance estimation R-package for
#' repeated surveys - useful for estimates of changes in quarterly and annual
#' averages}, \emph{Romanian Statistical Review} nr. 2 / 2016, pp. 17-28.
#' CONFERENCE: \emph{New Challenges for Statistical Software - The Use of R in
#' Official Statistics}, Bucharest, Romania, 7-8 April. \cr
#' 
#' Särndal, C.-E. and Lundström, S. (2005): \emph{Estimation in Surveys with
#' Nonresponse}, John Wiley and Sons, New York.
#' @keywords calibration
#' @encoding UTF8
"_PACKAGE"

lagVekter = function(
  modellen,
  brutto,
  pop = NULL,
  min_w = -Inf,
  max_w = Inf,
  totalReturn=0,
  maxiter=10,
  samplingWeights=NULL,
  returnwGross=FALSE, 
  ginvtol,
  boundstol
  )
{
  z = brutto

  lm_modell = lm(as.formula(modellen),data=z)


  if(is.numeric(pop) & is.vector(pop))
  {
    if(length(pop)==1)
    {
       totPop=pop
    }
    else
    {
      totPop=setTotal(pop,lmObject=lm_modell)
    }
  }
  else
  {
    if(is.null(pop))
    {
      tp = getTotal(z,lm_modell,samplingWeights)
    }
    else
    {
      tp = getTotal(pop,lm_modell)
    }

    if(totalReturn>1) return(tp)
    totPop =  tp$N*tp$colSum/tp$colN
  }
  if(totalReturn>0) return(totPop)



  x_model = model.matrix(lm_modell)


  r_model = model.frame(lm_modell)[1][,]



  a = ImpVekt(x_model,r_model ,totPop=totPop,totPopReturn=TRUE,w=samplingWeights, ginvtol = ginvtol)

  w0=a$vekt



  w = w0[r_model==1]
  x_netto = x_model[r_model==1,]

  minw_eps = boundstol * max(min_w,1)
  maxwPlusEps = (1+boundstol) * max_w   # Avoid Inf-Inf

  fortsett = (sum(w<(min_w+minw_eps)|w>maxwPlusEps )>0)
  iter=0
  if(fortsett & !is.null(samplingWeights)) stop("Limits&SamplingWeights not implemented")
  while(fortsett)
  {
    iter=iter+1
    wFixed = NA+w
    wFixed[wFixed>min_w] = NA
    wFixed[w<min_w] = min_w
    wFixed[w>max_w] = max_w
    w = ImpVektFixed(x_netto,totPop=a$totPop,wFixed=wFixed, ginvtol = ginvtol)

    fortsett = (sum(w<(min_w-minw_eps)|w>maxwPlusEps )>0)

    if(fortsett)
      if((iter>=maxiter)|(sum(w[!is.na(wFixed)]<(min_w-minw_eps)|w[!is.na(wFixed)]>maxwPlusEps )>0))
      {
        w = w0[r_model==1]
        fortsett = FALSE
      }
  }

  w1 = w0
  w1[r_model==1] = w
  if(returnwGross){
    return(list(w=w1,wGross=a$wGross))

  }
  w1
}


setTotalOld = function(total,lmObject)
{
  setTotal0 <- get0("setTotalFromCalibrateSSBuser", ifnotfound = FALSE) # Hack som gjør det mulig å bytte ut ...
  if(is.function(setTotal0)){
    return(setTotal0(total,lmObject))
  }
  x   = model.matrix(lmObject)[1,]
  x[1:length(x)] = NA
  varnames = names(total)[names(total) %in% names(x)]
  for(i in 1:length(varnames))
    x[names(x)==varnames[i]] = total[names(total)==varnames[i]]
  x
}


getTotalOld = function(data,lmObject,w=NULL)
{ 
  getTotal0 <- get0("getTotalFromCalibrateSSBuser", ifnotfound = FALSE) # Hack som gjør det mulig å bytte ut ...
  if(is.function(getTotal0)){
    return(getTotal0(data,lmObject,w))
  }
  x=model.frame(lmObject)
  x1=dim(x)[1]
  x2=dim(x)[2]
  d1=dim(data)[1]
  while(dim(x)[1]<x1+d1)
  {
      diffn = x1+d1-dim(x)[1]
      if(diffn>dim(x)[1]) x=rbind(x,x)
      else x=rbind(x,x[1:diffn,])
  }
  x[(x1+1):(x1+d1),]=NA
  varnames =names(data)[names(data) %in% names(x)]
  if(length(varnames)>0) for(i in 1:length(varnames))
    x[(x1+1):(x1+d1),names(x)==varnames[i]] = data[,names(data)==varnames[i],drop=FALSE]
  m=model.matrix(lmObject,data=x,na.action=NULL)
  m=m[(x1+1):(x1+d1), ,drop=FALSE]
  x=NULL
  if(is.null(w))
  {
    x$colSum =  colSums(m,na.rm=TRUE)
    x$colN   =  colSums(!is.na(m))
    x$N = dim(m)[1]
  } else
  {
    x$colSum =  colSums(w*m,na.rm=TRUE)
    x$colN   =  colSums(w*(!is.na(m)))
    x$N = sum(w)
  }
  x
}


sumList = function(x,xNew)
{
  for(i in 1:length(x)) x[[i]] =  x[[i]]+ xNew[[i]]
  x
}

#
# Copy of ginv from package "MASS"
# Only difference: Possibility to return singular values
#
#  + override input by ginv_tol_FromUser
#
my_ginv = function (X, tol = sqrt(.Machine$double.eps),singularReturn=FALSE)
{
  
    tolFromUser <- get0("ginv_tol_FromUser", ifnotfound = FALSE)
    if(tolFromUser){
      if(is.logical(tolFromUser))
        tol = sqrt(.Machine$double.eps)
      else{
        if(is.numeric(tolFromUser))
          tol = tolFromUser
      }
      #cat("ginv-tol set to",tol,"\n")
      #flush.console()
    }
  
    if (length(dim(X)) > 2L || !(is.numeric(X) || is.complex(X)))
        stop("'X' must be a numeric or complex matrix")
    if (!is.matrix(X))
        X <- as.matrix(X)
    Xsvd <- svd(X)
    if (is.complex(X))
        Xsvd$u <- Conj(Xsvd$u)
    Positive <- Xsvd$d > max(tol * Xsvd$d[1L], 0)

    if (all(Positive))
      invX = Xsvd$v %*% (1/Xsvd$d * t(Xsvd$u))  # new: "invX ="
    else if (!any(Positive))
      invX = array(0, dim(X)[2L:1L])            # new: "invX ="
    else invX = Xsvd$v[, Positive, drop = FALSE] %*% ((1/Xsvd$d[Positive]) *
      t(Xsvd$u[, Positive, drop = FALSE]))    # new: "invX ="
    if(!singularReturn) return(invX)
    list(invX=invX,singular=Xsvd$d)
}

#' Generate test data
#' 
#' Generate test data of eight quarters
#' 
#' 
#' @param n Number of observations within each quarter.
#' @return A data frame with the following variables: \item{id}{Sample unit
#' identifier} \item{year}{Year} \item{q}{Quarter} \item{month}{Month}
#' \item{R}{Response indicator} \item{age}{Age group} \item{sex}{Education
#' group} \item{famid}{Family identifier} \item{unemployed}{Unemployed}
#' \item{workforce}{In workforce}
#' @examples
#' 
#' # Generates data - two years
#' z = AkuData(3000) # 3000 in each quarter
#' 
#' @export AkuData
AkuData = function(n)
{
  #data("testDataBasis",envir=environment())
  testDataBasis = getTestDataBasis()
  ix=sample(1:dim(testDataBasis)[1],size=100+round(n/0.4),replace=T,prob=testDataBasis[,9])
  x=testDataBasis[ix,1:8]
  sstat = (x %% 1000) - 100
  sstat[sstat>500] = NaN
  sstat[apply(!is.na(sstat),2,cumsum)>n] = NaN
  rows = rowSums(!is.na(sstat))>0
  x = x[rows,]
  sstat = as.vector(sstat[rows,])
  id=as.vector(row(x))
  age = as.vector(x %/% 1000)
  R = as.numeric(sstat>0)
  q     = as.vector(1+ (col(x)-1) %%4)
  year  = as.vector(2014+ (col(x)-1) %/%4)
  month  = as.vector(((row(x)-1) %%3)) + 1 + (q-1)*3
  edu   = rep(sample(1:4,size=dim(x)[1],replace=T,prob=c(3,6,4,2)),dim(x)[2])
  sex   = rep(sample(c(0,1),size=dim(x)[1],replace=T),dim(x)[2])
  id = 1:dim(x)[1]
  famid  = rep(sample(1:60,size=dim(x)[1],replace=T),dim(x)[2]) + 100*(id %/%100)
  z=data.frame(id,year,q,month,R,age,sex,edu,famid)
  z =z[is.finite(sstat),]
  sstat = sstat[is.finite(sstat)]
  rownames(z) = NULL
  z$year      = factor(z$year)
  z$q         = factor(z$q)
  z$month     = factor(z$month)
  z$age      = factor(z$age)
  z$sex      = factor(z$sex)
  z$edu      = factor(z$edu)
  z$unemployed  = as.numeric(sstat==200)
  z$workforce   = as.numeric(sstat==100 | sstat==200)
  z
}

# stackoverflow questions 30357330
pkgEnvAkuData <- new.env(parent=emptyenv())
  if(!exists("testDataBasis", pkgEnvAkuData)) {
  data("testDataBasis", package="CalibrateSSB", envir=pkgEnvAkuData)
}

getTestDataBasis <- function() {
  pkgEnvAkuData[["testDataBasis"]]
}





#' testDataBasis
#' 
#' Data used by \code{\link{AkuData}}
#' 
#' @name testDataBasis
#' @docType data
#' @keywords datasets internal
NULL

#' Variance estimation for panel data
#'
#' Variance estimation of linear combinations of totals and ratios based on
#' output from wideFromCalibrate
#'
#' When denominator=NULL, only estimates for a single y-variable (numerator)
#' are calculated. When denominator is specified, estimates for numerator,
#' denominator and ratio are calculated. The default estimation type parameter,
#' "robustModel", is equation (12) in paper. "ssbAKU" is (16), "robustModelww"
#' is (9) and "robustModelGroup" and "robustModelGroupww" are cluster robust
#' variants based on \eqn{(w-1)^2} and \eqn{w^2} .
#'
#' @param x Output from wideFromCalibrate.
#' @param numerator y variable name or number.
#' @param denominator y variable name or number.
#' @param linComb Matrix defining linear combinations of waves.
#' @param linComb0 Linear combination matrix to be used prior to ratio
#' calculations.
#' @param estType Estimation type: "robustModel" (default), "ssbAKU",
#' "robustModelww", "robustModelGroup" or "robustModelGroupww" (see below)
#' @param leveragePower Power used when adjusting residuals using leverages.
#' @param group Extra variable name or number for cluster robust estimation.
#' @param returnCov Return covariance matrices instead of variance vectors.
#' @param usewGross Use wGross (if avaliable) instead of design weights to
#' adjust covariance matrix in the case of NA popTotals
#' @return \item{wTot}{Sum of weights} \item{estimates}{Ordinary estimates}
#' \item{linCombs}{Estimates of linear combinations}
#' \item{varEstimates}{Variance of estimates} \item{varLinCombs}{Variance of
#' estimates of linear combinations} When denominator is specified the above
#' output refer to ratios. Then, similar output for numerator and denominator
#' are also included.
#' 
#' @seealso \code{\link{CalibrateSSB}}, \code{\link{CalSSBobj}}, \code{\link{WideFromCalibrate}}, \code{\link{CalibrateSSBpanel}}. 
#' 
#' @examples
#'
#' # Generates data  - two years
#' z    = AkuData(3000)  # 3000 in each quarter
#' zPop = AkuData(10000)[,1:7]
#'
#' # Calibration and "WideFromCalibrate"
#' b = CalibrateSSB(z,calmodel="~ sex*age", partition=c("year","q"),
#'         popData=zPop, y=c("unemployed","workforce"))
#' bWide = WideFromCalibrate(b,CrossStrata(z[,c("year","q")]),z$id)
#'
#' # Define linear combination matrix
#' lc = rbind(LagDiff(8,4),PeriodDiff(8,4))
#' rownames(lc) = c("diffQ1","diffQ2","diffQ3","diffQ4","diffYearMean")
#' colnames(lc) = colnames(head(bWide$y[[1]]))
#' lc
#'
#' # Unemployed: Totals and linear combinations
#' d1=PanelEstimation(bWide,"unemployed",linComb=lc)  #
#'
#' # Table of output
#' cbind(tot=d1$estimates,se=sqrt(d1$varEstimates))
#' cbind(tot=d1$linCombs,se=sqrt(d1$varLinCombs))
#'
#' # Ratio: Totals and linear combinations
#' d=PanelEstimation(bWide,numerator="unemployed",denominator="workforce",linComb=lc)
#' cbind(tot=d$estimates,se=sqrt(d$varEstimates))
#' cbind(tot=d$linCombs,se=sqrt(d$varLinCombs))
#'
#' \dontrun{
#' # Calibration when som population totals unknown (edu)
#' # Leverages in output (will be used to adjust residuals)
#' # Cluster robust estimation (families/famid)
#' b2 = CalibrateSSB(z,popData=zPop,calmodel="~ edu*sex + sex*age",
#'            partition=c("year","q"), y=c("unemployed","workforce"),
#'            leverageOutput=TRUE)
#' b2Wide = WideFromCalibrate(b2,CrossStrata(z[,c("year","q")]),z$id,extra=z$famid)
#' d2 = PanelEstimation(b2Wide,"unemployed",linComb=lc,group=1,estType = "robustModelGroup")
#' cbind(tot=d2$linCombs,se=sqrt(d2$varLinCombs))
#' }
#'
#'
#' # Yearly mean before ratio calculation (linComb0)
#' # and difference between years (linComb)
#' g=PanelEstimation(bWide,numerator="unemployed",denominator="workforce",
#'     linComb= LagDiff(2),linComb0=Period(8,4))
#' cbind(tot=g$linCombs,se=sqrt(g$varLinCombs))
#'
#' @export PanelEstimation
PanelEstimation = function(x,numerator,denominator=NULL,linComb=matrix(0,0,n),linComb0=NULL,
                      estType="robustModel",leveragePower=1/2,group=NULL,returnCov=FALSE,usewGross=TRUE){
  if(class(x$w)[1]=="NULL"){
    z = vector("list",length(x))
    names(z) = names(x)
    n = dim(ListCbind(x[[1]]$y,numerator))[2]
    for(i in 1:length(x))
      z[[i]] = PanelEstimation(x[[i]],numerator,denominator,linComb,linComb0,estType,leveragePower,group,returnCov,usewGross)
    return(z)
  }
  if(is.null(denominator)){ # Enklere versjon av koden under
    m = length(numerator)
    y = ListCbind(x$y,numerator)
    n = dim(y)[2]
    nlc = dim(linComb)[1]
    w = RepCbind(x$w,m)

    if(usewGross & !is.null(x$wGross)){
      samplingWeights  = RepCbind(x$wGross,m)  # **** wGross -> samplingWeights
    } else {
      if(!is.null(x$samplingWeights))
        samplingWeights  = RepCbind(x$samplingWeights,m)
      else
        samplingWeights=NULL
    }
    if(!is.null(x$leverages))
      leverages  = RepCbind(x$leverages,m)
    else
      leverages = 0
    if(!is.null(x$leverages2))
      leverages2  =  RepCbind(x$leverages2,m)
    else
      leverages2 = 0
    resids  = ListCbind(x$resids,numerator)/(1-leverages)^leveragePower
    if(!is.null(x$resids2))
      resids2  = ListCbind(x$resids2,numerator)/(1-leverages2)^leveragePower
    else
      resids2 = NULL

  } else{
    n = dim(x$y[[numerator]])[2]
    nlc = dim(linComb)[1]
    y = cbind(x$y[[numerator]],x$y[[denominator]])
    w = cbind(x$w,x$w)
    if(usewGross & !is.null(x$wGross)){
      samplingWeights  = cbind(x$wGross,x$wGross) # **** wGross -> samplingWeights
    } else {
      if(!is.null(x$samplingWeights))
        samplingWeights  = cbind(x$samplingWeights,x$samplingWeights)
      else
        samplingWeights=NULL
    }
    if(!is.null(x$leverages))
      leverages  = cbind(x$leverages,x$leverages)
    else
      leverages = 0
    if(!is.null(x$leverages2))
      leverages2  = cbind(x$leverages2,x$leverages2)
    else
      leverages2 = 0
    resids  = cbind(x$resids[[numerator]],x$resids[[denominator]])/(1-leverages)^leveragePower
    if(!is.null(x$resids2))
      resids2  = cbind(x$resids2[[numerator]],x$resids2[[denominator]])/(1-leverages2)^leveragePower
    else
      resids2 = NULL
  }
  if(!is.null(group)){
    gr = group
    group = rowNoNA(data.matrix(x$extra[[gr]])) ##### First element.
    group2 = rowNoNA(data.matrix(x$extra[[gr]]),max) ##### Last element.
    eq = sum(as.numeric(!(group==group2),na.rm=TRUE))
    if(eq>0)
      warning(sprintf("Non-unique group detected. Last not equal first in %d cases. First used.",eq))
  }
  covTotals = TotalsWithCov(y,resids,w,estType,resids2,
                            samplingWeights=samplingWeights,group=group)
  a=NULL
  a$wTot = colSums(x$w,na.rm = TRUE)
  if(!is.null(samplingWeights))
    a$samplingWeightsTot = colSums(samplingWeights,na.rm = TRUE)

  if(is.null(denominator)){
    if(is.null(linComb0)) A = linComb
    else A = linComb %*% linComb0
    pEst = PanelEst(covTotals$totals,covTotals$covTotals,
                    A = A,returnCov=returnCov)
    a$estimates    =  pEst$totals[,1,drop=TRUE]
    if(nlc) a$linCombs     =  pEst$Atotals[,1,drop=TRUE]
    a$varEstimates =  pEst$varTotals
    if(nlc) a$varLinCombs  =  pEst$varAtotals
    return(a)
  }
  nn=n
  diag2n =  diag(1,2*n)
  rownames(diag2n) = colnames(y)
  if(is.null(linComb0)) {
    A = rbind(diag2n,cbind(linComb,matrix(0,dim(linComb)[1],n)),cbind(matrix(0,dim(linComb)[1],n),linComb))
    numerator=1:n
    names(numerator) = colnames(y)[1:n]
  } else {
    A2        = rbind(cbind(linComb0,matrix(0,dim(linComb0)[1],n)),cbind(matrix(0,dim(linComb0)[1],n),linComb0))
    linComb2  = linComb %*% linComb0
    AlinComb2 = rbind(cbind(linComb2,matrix(0,dim(linComb2)[1],n)),cbind(matrix(0,dim(linComb2)[1],n),linComb2))
    A = rbind(A2,AlinComb2)
    n = dim(linComb0)[1]
    nlc = dim(linComb2)[1]
    numerator=1:n
    names(numerator) = rownames(linComb0)
  }
  pEst = PanelEst(covTotals$totals,covTotals$covTotals,
                  A = A,
                  numerator=numerator,denominator=(n+1):(2*n), B=linComb,
                  returnCov=returnCov)
  a$estimates       =  pEst$ratios[,1,drop=TRUE]
  a$estimatesNum    =  pEst$totals[1:nn,1,drop=TRUE]
  a$estimatesDen    =  pEst$totals[(nn+1):(2*nn),1,drop=TRUE]
  if(nlc) a$linCombs      =  pEst$Bratios[,1,drop=TRUE]
  if(nlc) a$linCombsNum   =  pEst$Atotals[(2*n+1):(2*n+nlc),1,drop=TRUE]
  if(nlc) a$linCombsDen   =  pEst$Atotals[(2*n+nlc+1):(2*n+2*nlc),1,drop=TRUE]
  a$varEstimates    =  pEst$varRatios
  a$varEstimatesNum =  TakeIndexBoth(pEst$varTotals,1:nn)
  a$varEstimatesDen =  TakeIndexBoth(pEst$varTotals,(nn+1):(2*nn))
  if(nlc) a$varLinCombs     =  pEst$varBratios
  if(nlc) a$varLinCombsNum  =  TakeIndexBoth(pEst$varAtotals,(2*n+1):(2*n+nlc))
  if(nlc) a$varLinCombsDen  =  TakeIndexBoth(pEst$varAtotals,(2*n+nlc+1):(2*n+2*nlc))
  a
}



PanelEst = function(totals,covTotals,A=matrix(0,0,length(totals)),numerator=integer(0),denominator=integer(0),
                    B=diag(1,length(numerator)),
                    returnCov=FALSE,
                    rationames=names(numerator))
{
  #### Level 1: Input variables
  #### Level 2: Lin.comb of input
  Atotals    = A %*% totals
  covAtotals = A %*% covTotals %*% t(A)


  #### Level 3: Ratios of lin.comb
  # Variansen til X/Y beregnes som variansen til X/y - Yx/yy
  rNum = Atotals[numerator,,drop=FALSE]    # teller-estimater (x)
  rDen = Atotals[denominator,,drop=FALSE]  # nevner-estimater (y)
  D = matrix(0,nrow=length(rNum),ncol=dim(A)[1]) # D genererer "X/y - Yx/yy"
  if(dim(D)[1]) for(i in 1:dim(D)[1]){
    D[i,numerator[i]]   = 1/rDen[i]  # X blir multiplisert med "1/y"
    D[i,denominator[i]] = -rNum[i]/(rDen[i])^2 # Y blir multiplisert med "x/yy"
  }
  ratios  = rNum/rDen
  rownames(ratios) = rationames
  rownames(D) = rationames
  covRatios = D %*% covAtotals %*% t(D)

  #### Level 4: Lin.comb of ratios
  Bratios = B %*% ratios
  covBratios = B %*% covRatios %*% t(B)

  if(returnCov) return(list(totals=totals,Atotals=Atotals,ratios=ratios,Bratios=Bratios,
                            varTotals=covTotals,varATotals=covAtotals,varRatios=covRatios,varBratios=covBratios))
  list(totals=totals,Atotals=Atotals,ratios=ratios,Bratios=Bratios,varTotals=diag(covTotals),varAtotals=diag(covAtotals),
       varRatios=diag(covRatios),varBratios=diag(covBratios))
}



TotalsWithCov = function(y,resids,w,estType="robustModel",   #dummy=!is.na(w)
                         resids2=NULL,returnNr=FALSE,returnNr1=FALSE,
                         samplingWeights=NULL,dummyGross=!is.na(w),dummyNet=!is.na(resids), ...){
  force(dummyGross) # dummy is created (lazy evaluation)
  force(dummyNet) # dummy is created (lazy evaluation)
  a=NULL
  y[is.na(y)]=0
  w[is.na(w)]=0
  resids[is.na(resids)]=0
  a$totals = matrix(colSums(w*y),nrow=dim(y)[2]) # T is column vector
  rownames(a$totals) = colnames(y)

  a$covTotals = MakeCovTotals(resids,w,dummyNet,estType,...)


  if(!is.null(resids2)){
    resids2[is.na(resids2)]=0
    if(is.null(samplingWeights)){
      samplingWeights = t(matrix(colSums(w)/colSums(dummyGross),dim(w)[2],dim(w)[1]))
    }
    else samplingWeights[is.na(samplingWeights)] = 0
    v=w/samplingWeights
    v[is.na(v)]=0
    if(estType=="ssbAKU") estType = "robustModel"   # Obs here
    covTotalsNr  = MakeCovTotals(resids2*samplingWeights,v,dummyNet,estType,...)
    covTotalsNr1 = MakeCovTotals(resids*samplingWeights,v,dummyNet,estType,...)
    a$covTotals = a$covTotals - covTotalsNr1 + covTotalsNr
    if(returnNr)   a$covTotalsNr  = covTotalsNr
    if(returnNr1)  a$covTotalsNr1 = covTotalsNr1
  }
  a
}


MakeCovTotals  = function(e,w,dummy,estType,group=NULL){ # e instead of resids
  if(estType=="ssbAKU") {
    n = t(dummy) %*% dummy
    We  = w*e
    mWe = matrix(colSums(We)/diag(n),nrow=1)
    covTotals = n/(n-1) * (t(We)%*%We - n*t(mWe)%*%mWe)
  }

  if(estType=="robustModel"){
    we = (w-1)*e
    covTotals = t(we)%*%we
    we = sqrt(pmax(w-1,0))*e    ### Negativ (w-1) settes til 0
    covTotals = covTotals + t(we)%*%we
  }

  if(estType=="robustModelww"){
    we = w*e
    covTotals = t(we)%*%we
  }
  if(estType=="robustModelGroup"|estType=="robustModelGroupww"){
    we = data.matrix(aggregate(w*e,list(group),sum)[,-1,drop=FALSE])
    if(estType=="robustModelGroupww"){
      covTotals = t(we)%*%we
    } else{
      we1 = data.matrix(aggregate((w-1)*e,list(group),sum)[,-1,drop=FALSE])
      covTotals = t(we)%*%we1
    }
  }
  covTotals[!is.finite(covTotals)]=0 # Avoid problems when "n-1=0" and n=0
  covTotals
}


rowNoNA = function(x,maxmin=min){
  colx = col(x)
  colx[is.na(x)] = NA
  element = apply(colx,1,function(x) maxmin(x,na.rm=T))
  x[cbind(1:dim(x)[1],element)]
}

TakeIndexBoth = function(x,index){
  if(is.matrix(x)) return(x[index,index,drop=FALSE])
  x[index]
}


seq_ = function(a,b) seq(a,b,length = max(0,b-a+1))

MyDiag = function(n,k=0){
  x = diag(n)
  z = matrix(0,n,n)
  a  = max(1+k,1)
  b  = min(n+k,n)
  a2 = max(1-k,1)
  b2 = min(n-k,n)
  z[,seq_(a,b)] = x[,seq_(a2,b2)]
  z
}





#' Creation of linear combination matrices
#'
#' Create matrices for changes (LagDiff), means (Period) and mean changes
#' (PeriodDiff).
#'
#'
#' @aliases LinCombMatrix PeriodDiff Period LagDiff
#' @param n Number of variables
#' @param period Number of variables involved in each period
#' @param lag Lag used for difference calculation
#' @param k Shift the start of each period
#' @param takeMean Calculate mean over each period (sum when FALSE)
#' @param removerows Revove incomplete rows
#' @param overlap Overlap between periods (moving averages)
#' @return Linear combination matrix
#' @note It can be useful to add row names to the resulting matrix before
#' further use.
#' @examples
#'
#' # We assume two years of four quarters (n=8)
#'
#' # Quarter to quarter differences
#' LagDiff(8)
#'
#' # Changes from same quarter last year
#' LagDiff(8,4)
#'
#' # Yearly averages
#' Period(8,4)
#'
#' # Moving yearly averages
#' Period(8,4,overlap=TRUE)
#'
#' # Difference between yearly averages
#' PeriodDiff(8,4) # Also try n=16 with overlap=TRUE/FALSE
#'
#' # Combine two variants and add row names
#' lc = rbind(LagDiff(8,4),PeriodDiff(8,4))
#' rownames(lc) = c("diffQ1","diffQ2","diffQ3","diffQ4","diffYearMean")
#' lc
#'
#' @export LinCombMatrix
LinCombMatrix = function(n,period=NULL,lag=NULL,k=0,takeMean=TRUE,removerows=TRUE,overlap=FALSE){
  if(is.null(period)){
    if(is.null(lag)) x = diag(n)
    else x = LagDiff(n,lag=lag,removerows=removerows)
  } else{
    if(is.null(lag)) x = Period(n=n,period=period,k=k,takeMean=takeMean,removerows=removerows,overlap=overlap)
    else x = PeriodDiff(n=n,period=period,lag=lag,k=k,takeMean=takeMean,removerows=removerows,overlap=overlap)
  }
  x
}



#' @rdname LinCombMatrix
#' @encoding UTF8
#' @export
#'
LagDiff= function(n,lag=1,removerows=TRUE){
  m=MyDiag(n,lag) - diag(n)
  if(removerows) m = m[rowSums(m)==0 & rowSums(abs(m))==2 , ,drop=FALSE]
  m
}

#' @rdname LinCombMatrix
#' @encoding UTF8
#' @export
#'
Period = function(n,period=1,k=0,takeMean=TRUE,removerows=TRUE,overlap=FALSE){
  x =matrix(0,n,n)
  for(i in seq_(1,period) ) x = x+MyDiag(n,k+i-1)
  if(removerows) x = x[rowSums(x)==period, ,drop=FALSE]
  if(!overlap)   x = x[((-1+1:dim(x)[1])%%period)==0, ,drop=FALSE]
  if(takeMean)   x = x/period
  x
}

#' @rdname LinCombMatrix
#' @encoding UTF8
#' @export
#'
PeriodDiff = function(n,period=1,lag=period,k=0,takeMean=TRUE,removerows=TRUE,overlap=FALSE){
  a = Period(n=n,period=period,k=k,takeMean=FALSE,removerows=FALSE,overlap=TRUE)
  b = Period(n=n,period=period,k=k+lag,takeMean=FALSE,removerows=FALSE,overlap=TRUE)
  x = b-a
  if(removerows) x = x[rowSums(x)==0 & rowSums(abs(x))==2*period , ,drop=FALSE]
  if(!overlap)   x = x[((-1+1:dim(x)[1])%%period)==0, ,drop=FALSE]
  if(takeMean)   x = x/period
  x
}






RepCbind = function(x,n){
  z = NULL
  for(i in seq_len(n)) z=cbind(z,x)
  z
}

ListCbind = function(x,elements,sep="-"){
  z = NULL
  n = length(elements)
  for(i in seq_len(n)){
    z1 = x[[elements[i]]]
    colnames(z1) = paste(elements[i],colnames(z1),sep=sep)
    z  = cbind(z,z1)
  }
  z
}

suppressWarnings(RNGversion("3.5.0"))
set.seed(1234)
z    <- AkuData(100)  
zPop <- AkuData(200)[,1:7]
z$samplingWeights = sample(10000:100000,NROW(z))

z$sex = as.character(z$sex)
zPop$age = as.character(zPop$age)


Csum <- function(a){
  x <- as.numeric(as.matrix(a))
  x <- x[!is.na(x)]
  c(length(x),sum(x),sum(x^2))
}

#z = tibble::as_tibble(z)
#zPop = tibble::as_tibble(zPop)


test_that("Examples", {
  
  
  #### CalibrateSSB Examples
  
  # Calibration using "survey"
  a <- CalibrateSSB(z, calmodel = "~ sex*age",
                    partition = c("year","q"),  # calibrate within quarter
                    popData = zPop, y = c("unemployed","workforce"),
                    by = c("year","q")) # Estimate within quarter
  expect_identical(Csum(a$popTotals),c(64,20096,32956616))
  expect_equal(Csum(a$w),c(800,1600,4428.729572165))
  expect_equal(Csum(a$estTM),c(32, 17414.751745, 32658559.6096))
  
  
  expect_identical(a$popTotals, CalibrateSSB(z, calmodel = "~ sex*age",
                    partition = c("year","q"),  # calibrate within quarter
                    popData = zPop, y = c("unemployed","workforce"),
                    by = c("year","q"), onlyTotals = TRUE))
  
  
  # Calibration, no package, popTotals as input
  b <- CalibrateSSB(z, popTotals=a$popTotals, calmodel="~ sex*age",
                    partition = c("year","q"), usePackage = "none", y = c("unemployed","workforce"))
  
  expect_identical(a$popTotals, b$popTotals)
  expect_equal(a$w, b$w)
  expect_equal(colSums(a$estTM[,3:4]), colSums(b$estTM))
  
  
  expect_identical(a$popTotals, CalibrateSSB(z, popTotals=a$popTotals, calmodel="~ sex*age",
                    partition = c("year","q"), usePackage = "none", 
                    y = c("unemployed","workforce"), onlyTotals = TRUE))
  
  
  #### PanelEstimation Examples
  
  bWide = WideFromCalibrate(b,CrossStrata(z[,c("year","q")]),z$id)
  
  # Define linear combination matrix
  lc = rbind(LagDiff(8,4),PeriodDiff(8,4))
  rownames(lc) = c("diffQ1","diffQ2","diffQ3","diffQ4","diffYearMean")
  colnames(lc) = colnames(head(bWide$y[[1]]))

  # Unemployed: Totals and linear combinations
  d1=PanelEstimation(bWide,"unemployed",linComb=lc)  
  
  expect_equal(range(d1$wTot), c(200,200))
  expect_equal(Csum(c(d1$estimates, d1$linCombs, d1$varEstimates, d1$varLinCombs)),
               c(26, 113.4170829057, 1674.9380089581))
  
  d=PanelEstimation(bWide,numerator="unemployed",denominator="workforce",linComb=lc)
  
  expect_equal(Csum(unlist(d)), c(86, 3493.652535091372, 535118.167577150976))
  
  expect_warning(b2 <- CalibrateSSB(z,popData=zPop,calmodel="~ edu*sex + sex*age",
                    partition=c("year","q"), y=c("unemployed","workforce"),
                    leverageOutput=TRUE))
  

  expect_equal(Csum(log(unlist(b2)+1)), c(7352, 2206.516235572093, 3617.600635213678))
  
  
  b2Wide = WideFromCalibrate(b2,CrossStrata(z[,c("year","q")]),z$id,extra=z$famid)
  d2 = PanelEstimation(b2Wide,"unemployed",linComb=lc,group=1,estType = "robustModelGroup")
  
  expect_equal(Csum(unlist(d2)), c(42, 3329.254745316503, 642035.689654916176)) 
  
  g=PanelEstimation(bWide,numerator="unemployed",denominator="workforce",
                    linComb= LagDiff(2),linComb0=Period(8,4))
  
  expect_equal(Csum(unlist(g)), c(50, 3208.179712767641, 522114.947009256517))
  
  expect_equal(Csum(c(g$varEstimates, g$varLinCombs)),c(3, 2.554084342077243e-04, 2.771624841133405e-08))
  
})

test_that("samplingWeights", {
  
  a_popTotals = CalibrateSSB(z, calmodel = "~ age*sex",    # instead of sex*age
               partition = c("year","q"),  # calibrate within quarter
               popData = zPop, y = c("unemployed","workforce"),
               by = c("year","q"), onlyTotals = TRUE)
  
  # Calibration, "survey", popTotals as input
  bS <- CalibrateSSB(z, popTotals=a_popTotals, calmodel="~ sex*age",
                     partition = c("year","q"), usePackage = "survey", y = c("unemployed","workforce"),
                     samplingWeights ="samplingWeights")
  
  # Calibration, no package, popTotals as input
  bN <- CalibrateSSB(z, popTotals=a_popTotals, calmodel="~ sex*age",
                     partition = c("year","q"), usePackage = "none", y = c("unemployed","workforce"),
                     samplingWeights ="samplingWeights")
  
  expect_equal(Csum(log(unlist(bS)+1.2)), c(3828,1877.483679157398, 2552.801866633156))
  expect_equal(bS,bN)
  
  
  # Calibration, no package, popTotals as input
  bN <- CalibrateSSB(z, popData=zPop, calmodel="~ sex*age",
                      partition = c("year","q"), usePackage = "none", y = c("unemployed","workforce"),
                      samplingWeights ="samplingWeights")
  
  expect_equal(bS,bN)
  
  
  bS <- CalibrateSSB(z, calmodel="~ sex*age",
                      partition = c("year","q"), usePackage = "survey", y = c("unemployed","workforce"),
                      samplingWeights ="samplingWeights")
  
  
  bN <- CalibrateSSB(z, calmodel="~ sex*age",
                     partition = c("year","q"), usePackage = "none", y = c("unemployed","workforce"),
                     samplingWeights ="samplingWeights")
  
  expect_equal(Csum(log(unlist(bS)+1.2)), c(3828,9020.196061889004, 92562.450246460881))
  expect_equal(bS,bN)
  
  expect_identical(bS$popTotals, CalibrateSSB(z, calmodel="~ sex*age",
                  partition = c("year","q"), usePackage = "none", y = c("unemployed","workforce"),
                  samplingWeights ="samplingWeights",onlyTotals = TRUE))
  
  expect_identical(bS$popTotals, CalibrateSSB(z, calmodel="~ sex*age",
                  partition = c("year","q"), usePackage = "survey", y = c("unemployed","workforce"),
                  samplingWeights ="samplingWeights",onlyTotals = TRUE))

  
  zPop$age[sample(NROW(zPop),100)] = "4"
  
  expect_error(CalibrateSSB(z, popData=zPop, calmodel="~ sex*age",
               partition = c("year","q"), usePackage = "survey", y = c("unemployed","workforce"),
               samplingWeights ="samplingWeights"))
  
  zPop$age[zPop$age == "4"] = NA
  
  
  # Calibration, "survey", popTotals as input
  bS <- CalibrateSSB(z, popData=zPop, calmodel="~ sex*age",
                     partition = c("year","q"), usePackage = "survey", y = c("unemployed","workforce"),
                     samplingWeights ="samplingWeights")
  
  # Calibration, no package, popTotals as input
  bN <- CalibrateSSB(z, popData=zPop, calmodel="~ sex*age",
                     partition = c("year","q"), usePackage = "none", y = c("unemployed","workforce"))
  
  expect_equal(Csum(log(unlist(bS)+1.2)), c(3828, 1878.749175681913, 2554.80550645861))
  expect_equal(Csum(log(unlist(bN)+1)), c(3828, 1299.31846833678, 2415.705455668723))
  
  expect_warning(bS <- CalibrateSSB(z,popData=zPop,calmodel="~ edu*sex + sex*age",
               partition=c("year","q"), y=c("unemployed","workforce"),
               leverageOutput=TRUE, samplingWeights ="samplingWeights"))
  
  
  expect_equal(Csum(log(unlist(bS)+1.1)), c(7352, 2728.768354842984, 3667.182228662961)) 
  
  expect_identical(bS$popTotals, suppressWarnings(
    CalibrateSSB(z,popData=zPop,calmodel="~ edu*sex + sex*age",
                 partition=c("year","q"), y=c("unemployed","workforce"),
                 leverageOutput=TRUE, samplingWeights ="samplingWeights",
                 usePackage = "none", onlyTotals = TRUE)))
  
  
  bSWide = WideFromCalibrate(bS,CrossStrata(z[,c("year","q")]),z$id)
  # Define linear combination matrix
  lc = rbind(LagDiff(8,4),PeriodDiff(8,4))
  
  # Unemployed: Totals and linear combinations
  d1=PanelEstimation(bSWide,numerator="unemployed",denominator="workforce",linComb=lc, leveragePower = 1)  
  
  expect_equal(Csum(sqrt(unlist(d1)+15)), c(102, 817.3757246497714, 8722.4130549911861))
  
  
  # MERK: Variabelnavn i CalibrateSSB og vektor-input i WideFromCalibrate bør forklares bedre 
  
  expect_warning(d2 <- CalibrateSSBpanel(z, popData=zPop,calmodel="~ sex*edu + age*sex",  # edu*sex + sex*age" over
                        partition=c("year","q"), 
                        y=c("unemployed","workforce"),leverageOutput=TRUE,samplingWeights ="samplingWeights",
                        wave=c("year","q"), id = "id", usePackage = "none",
                        numerator="unemployed",denominator="workforce",linComb=lc, leveragePower = 1))
  
  expect_equal(d1,d2)
  
})


test_that("extra", {
  
  lc = rbind(LagDiff(8,4),PeriodDiff(8,4))
  
  #tt = function(x)  tibble::as_tibble(x)
  
  z$yq = CrossStrata(z[,c("year","q")])
  zPop$yq = CrossStrata(zPop[,c("year","q")])
  
  a <- CalibrateSSBpanel(z, popData=zPop,calmodel="~ sex",  # edu*sex + sex*age" over
                          partition=c("yq"), 
                          y=c("unemployed"),leverageOutput=TRUE,samplingWeights ="samplingWeights",
                          wave=c("yq"), id = "id", usePackage = "none",
                          numerator="unemployed",linComb=lc, leveragePower = 1)

  expect_equal(Csum(log(unlist(a)+ 15)), c(34, 114.9679972860836, 479.3773963999))
  
  
  
  a <- CalibrateSSB(z, calmodel = "~ sex*age",
                    partition = c("year","q"),  # calibrate within quarter
                    popData = zPop, y = c("unemployed","workforce"),
                    by = c("year")) # Estimate within quarter
  
    
  expect_equal(Csum(log(unlist(a)+ 1)), c(3832, 1307.354796766325, 2450.963556820278))
  
  
  
})



if(require(ReGenesees)){ test_that("ReGenesees", {
  z$sex = as.factor(z$sex)
  zPop$age = as.factor(zPop$age)
  
  
  # Calibration, no package, popData as input
  bN <- CalibrateSSB(z, popData=zPop, calmodel="~ sex*age",
                     partition = c("year","q"), usePackage = "none", y = c("unemployed","workforce"),
                     by = c("year","q"),
                     samplingWeights ="samplingWeights")
  
  # Calibration, "ReGenesees", popData as input
  bR <- CalibrateSSB(z, popData=zPop, calmodel="~ sex*age",
                     partition = c("year","q"), usePackage = "ReGenesees", y = c("unemployed","workforce"),
                     by = c("year","q"),
                     samplingWeights ="samplingWeights")

  expect_equal(bN$w, bR$w)
  expect_equal(as.data.frame(bR$popTotals[,3:8]),bN$popTotals[,3:8])
  estTM=merge(bR$estTM,bN$estTM)
  expect_equal(estTM[,"Total.workforce"],estTM[,"workforce"])
  
  
  # Residuals different when samplingWeights
  
  
  # Calibration, no package, popData as input, no samplingWeights
  bN <- CalibrateSSB(z, popData=zPop, calmodel="~ sex*age",
                     partition = c("year","q"), usePackage = "none", y = c("unemployed","workforce"),
                     by = c("year","q"))
  
  # Calibration, "ReGenesees", popData as input, no samplingWeights
  bR <- CalibrateSSB(z, popData=zPop, calmodel="~ sex*age",
                     partition = c("year","q"), usePackage = "ReGenesees", y = c("unemployed","workforce"),
                     by = c("year","q"))
  
  expect_equal(bN$resids[,2], bR$resids[,2])
  
})}

#   wregw i impvekt bør endres ...bereging av "*" mellom matrise og vektor
#   Kan se ut til at det blir riktig p.g.a beregningsrekkefølge, men det bør endres ....
#



#' Calibration weighting and estimation
#' 
#' Compute weights by calibration and corresponding estimates, totals and
#' residuals
#' 
#' When popTotals as input is NULL, population totals are computed from popData
#' (when available) or from grossSample. Some elements of popTotals may be
#' missing (not allowed when using ReGenesees). When using "ReGenesees", both
#' weiging and estimation are done by that package. When using "survey", only
#' calibration weiging are done by that package.
#' The parameters \code{wave}, \code{id} and \code{extra} have no effect on the 
#' computations, but result in extra elements in output 
#' (to be used by WideFromCalibrate() later).  
#' 
#' 
#' @encoding UTF8
#' 
#' @param grossSample Input data, typically a data frame, tibble, or data.table. 
#'                    If `grossSample` is not a classic data frame, 
#'                    it will be coerced to one internally. 
#' @param calmodel Formula defining the linear structure of the calibration
#' model.
#' @param response Variable name of response indicator (net sample when 1).
#' @param popTotals Population totals (similar to population totals as output).
#'                  If `popTotals` is not a classic data frame, 
#'                  it will be coerced to one internally.              
#' @param y Names of variables of interest. Can be a list similar to "by"
#' below.
#' @param by Names of the variables that define the "estimation domains". If
#' NULL (the default option) or NA estimates refer to the whole population. Use
#' list for multiple specifications (resulting in list as output).
#' @param partition Names of the variables that define the "calibration
#' domains" for the model. NULL (the default) implies no calibration domains.
#' @param lRegmodel Formula defining the linear structure of a logistic
#' regression model.
#' @param popData Population data, typically a data frame, tibble, or data.table. 
#'                If `popData` is not a classic data frame, 
#'                it will be coerced to one internally. 
#' @param samplingWeights Name of the variable with initial weights for the
#' sampling units.
#' @param usePackage Specifying the package to be used: "survey" (the default),
#' "ReGenesees" or "none".
#' @param bounds Bounds for the calibration weights. When ReGenesees: Allowed
#' range for the ratios between calibrated and initial weights. The default is
#' c(-Inf,Inf).
#' @param calfun The distance function for the calibration process; the default
#' is 'linear'.
#' @param onlyTotals When TRUE: Only population totals are returned.
#' @param onlyw When TRUE: Only the calibrated weights are returned.
#' @param uselRegWeights When TRUE: Weighted logistic regression is performed
#' as a first calibration step.
#' @param ids Name of sampling unit identifier variable.
#' @param residOutput Residuals in output when TRUE. FALSE is default.
#' @param leverageOutput Leverages in output when TRUE. FALSE is default.
#' @param yOutput y in output when TRUE. FALSE is default.
#' @param samplingWeightsOutput samplingWeights in output when TRUE. FALSE is
#' default.
#' @param dropResid2 When TRUE (default) and when no missing population totals
#' - only one set of residuals in output.
#' @param wGrossOutput wGross in output when TRUE (default) and when NA
#' popTotals.
#' @param wave Time or another repeat variable (to be included in output).
#' @param id Identifier variable (to be included in output).
#' @param extra Variables for the extra dataset (to be included in output).
#' @param allowNApopTotals When TRUE missing population totals are allowed.
#' Results in error when FALSE and warning when NULL.
#' @param partitionPrint When TRUE partition progress is printed. 
#' Automatic decision when NULL (about 1 min total computing time).  
#' 
#' @param ginvtol Tolerance parameter for the generalized inverse used when
#'   `usePackage = "none"` and also for the computation of `resids`
#'   when `resids_by_lm = FALSE` and `leverageOutput = FALSE`. The default
#'   (`1e-06`) is chosen with robustness of the function in mind, beyond
#'   machine precision effects. A more standard value is `1.5e-08`,
#'   similar to `ginv()` in the MASS package.
#'   
#' @param boundstol Tolerance parameter used in iterations involving bounds
#'   when `usePackage = "none"`.
#' 
#' @param resids_by_lm When `FALSE` (default) and `leverageOutput = FALSE`, 
#'   the computation of `resids` is based on a generalized inverse.
#'   Otherwise, the computations go via the `lm()` function.
#' 
#' @param ... Further arguments sent to underlying functions.
#' @return Unless onlyTotals or onlyw is TRUE, the output is an object of class calSSB. That is, a list with 
#' elements: \item{popTotals}{Population totals.} \item{w}{The calibrated
#' weights.} \item{wGross}{Calibrated gross sample weights when NA popTotals.}
#' \item{estTM}{Estimates (with standard error).} \item{resids}{Residuals,
#' reduced model when NA popTotals.} \item{resids2}{Residuals, full model.}
#' \item{leverages}{Diagonal elements of hat-matrix, reduced model when NA
#' popTotals.} \item{leverages2}{Diagonal elements of hat-matrix, full model.}
#' \item{y}{as input} \item{samplingWeights}{as input}
#' \item{wave}{as input or via CrossStrata}
#' \item{id}{as input}
#' \item{extra}{as input}
#' 
#' @export
#' @importFrom methods formalArgs
#' @importFrom stats aggregate as.formula binomial glm lm lm.influence model.frame model.matrix predict resid update weights
#' @importFrom utils head tail capture.output
#' @importFrom survey svydesign calibrate
#' 
#' @seealso \code{\link{CalSSBobj}}, \code{\link{WideFromCalibrate}},  \code{\link{PanelEstimation}}, \code{\link{CalibrateSSBpanel}}.
#' 
#' @examples
#' 
#' # Generates data  - two years
#' z    <- AkuData(3000)  # 3000 in each quarter
#' zPop <- AkuData(10000)[,1:7]
#' 
#' # Calibration using "survey"
#' a <- CalibrateSSB(z, calmodel = "~ sex*age",
#'                  partition = c("year","q"),  # calibrate within quarter
#'                  popData = zPop, y = c("unemployed","workforce"),
#'                  by = c("year","q")) # Estimate within quarter
#' head(a$w) # calibrated weights
#' a$estTM   # estimates
#' a$popTotals   # popTotals used as input below
#' 
#' 
#' # Calibration, no package, popTotals as input
#' b <- CalibrateSSB(z, popTotals=a$popTotals, calmodel="~ sex*age",
#'       partition = c("year","q"), usePackage = "none", y = c("unemployed","workforce"))
#' max(abs(a$w-b$w)) # Same weights as above
#' 
#' print(a)
#' print(b)
#' 
#' \dontrun{
#' require(ReGenesees)
#' # Calibration and estimation via ReGenesees
#' CalibrateSSB(z, calmodel = "~ sex*age",
#'              partition = c("year","q"),  # calibrate within quarter
#'              popData = zPop, usePackage = "ReGenesees",
#'              y = c("unemployed","workforce"),
#'              by = c("year","q")) # Estimate within quarter
#' }
#' 
CalibrateSSB = function(grossSample,calmodel=NULL,response="R",popTotals=NULL,y=NULL,by = NULL,partition=NULL,lRegmodel=NULL,
                        popData=NULL,samplingWeights=NULL,
                        usePackage="survey",bounds=c(-Inf,Inf),calfun="linear",
                        onlyTotals=FALSE,onlyw=FALSE,uselRegWeights=FALSE,ids=NULL,
                        residOutput = TRUE,
                        leverageOutput = FALSE,
                        yOutput = TRUE,
                        samplingWeightsOutput = FALSE,
                        dropResid2 = TRUE,
                        wGrossOutput = TRUE,
                        wave=NULL,
                        id=NULL,
                        extra=NULL,
                        allowNApopTotals = NULL,
                        partitionPrint = NULL,
                        ginvtol = 1e-06,
                        boundstol = 1e-06,
                        resids_by_lm = FALSE, 
                        ...){
  
  grossSample <- as.data.frame(grossSample)
  
  if (!is.null(popData)) {
    popData <- as.data.frame(popData)
  }
  
  if (!is.null(popTotals)) {
    popTotals <- as.data.frame(popTotals)
  }
  
  #if(hasArg("residOutput"))
  #  warning("residOutput is an old argument not in use")
  timeLimit = 60
  if(!is.null(allowNApopTotals)){
    checkAnyNApopTotals = !allowNApopTotals
  } else{
    checkAnyNApopTotals = TRUE
  }
    
  if(leverageOutput) residOutput = TRUE
  if(is.null(y)) residOutput = FALSE
  if(residOutput & is.list(y)) stop("residOutput & is.list(y) NOT IMPLEMENTED")
  if(yOutput & is.list(y)) stop("yOutput & is.list(y) NOT IMPLEMENTED")

  n = NROW(grossSample)
  R = grossSample[,response]

  if(is.null(samplingWeights)){
    samplingWeights = "Wei5652503017"
    grossSample[,samplingWeights] = 1
    samplingW = NULL
  } else samplingW = grossSample[[samplingWeights]]

  if(is.null(popData)){
    popData = grossSample
    wPop = samplingW
  } else wPop=NULL  # Avoid copy?


  ######## START ReGenesees
  if(tolower(usePackage)=="regenesees"){
    if(!is.null(lRegmodel))
      stop("lRegmodel when ReGenesees NOT IMPLEMENTED")
    #if(residOutput)
    #  stop("residOutput when ReGenesees NOT IMPLEMENTED")
    #if(yOutput)
    #  stop("yOutput when ReGenesees NOT IMPLEMENTED")
    #if(samplingWeightsOutput)
    #  stop("samplingWeightsOutput when ReGenesees NOT IMPLEMENTED")
    if(is.null(ids)){
      ids = "ids5652503017"
      grossSample[,ids] = 1:n
    }

    retur =CalibratePackageReGenesees(netSample=grossSample[R==1,],calmodel=calmodel,popTotals=popTotals,y=y,
                                          by=by,partition=partition,
                                          popData=popData,samplingWeights=samplingWeights,bounds=bounds,calfun=calfun,
                                          onlyTotals=onlyTotals,ids=ids,...)
    w = retur$w
    retur$w = rep(0,n)
    retur$w[R==1] = w
    if(onlyw) return(retur$w)
    resids = retur$resids
    retur$resids = matrix(NaN,n,dim(resids)[2])
    retur$resids[R==1,] = resids
    #return(retur)
    if(yOutput) retur$y = grossSample[,y,drop=FALSE]
    if(samplingWeightsOutput) 
      retur$samplingWeights <- grossSample[, samplingWeights, drop = FALSE]
    
    #if(!residOutput | !yOutput)
    #  return(retur)
    #return(structure(retur, class = "calSSB", n=n, nY=length(y)))
  }######## END ReGenesees
  else {


  bigStrataLevels=NULL
  createPopTotals = is.null(popTotals)
  partitionPop =FALSE
  bigStrataPop = rep(1,NROW(popData))

  if(!is.null(partition))  {
    partitionPop = sum(partition %in% names(popData))>0

    if(partitionPop & createPopTotals){
      cs = CrossStrata(grossSample[,partition],returnb=TRUE,asNumeric=TRUE,byExtra=popData[,partition])
      bigStrataPop = cs$aExtra
    } else {
      cs = CrossStrata(grossSample[,partition],returnb=TRUE,asNumeric=TRUE)
    }

    bigStrata       = cs$a
    bigStrataLevels = cs$b
    if(length(bigStrataLevels)==1) names(bigStrataLevels) = partition # Hindre at navnet blir "by"
  }
  else bigStrata = rep(1,n)

  nBig    = max(bigStrata)

  calModel=NULL
  lRegModel=NULL
  if(!is.null(calmodel))   calModel = update(as.formula(calmodel),paste(response,"~."))
  if(!is.null(lRegmodel)) lRegModel = update(as.formula(lRegmodel),paste(response,"~."))



  if(!createPopTotals){
    if(is.vector(popTotals)) popTotals = matrix(popTotals,nrow = 1,dimnames=list(" ",names(popTotals)))
    popTotalsInput = popTotals
    if(nBig>1 & NROW(popTotals)>1) popTotalsInput = mergeSort(popTotals,bigStrataLevels,remov=TRUE)
    popTotalsInput = as.matrix(popTotalsInput)
    popTotals=NULL
  }

  w = rep(NaN,n)
  if(wGrossOutput)   wGross = rep(NaN,n)

  if(residOutput){
    e1 = NaN+grossSample[,y,drop=FALSE]
    e2=e1
    if(resids_by_lm) {
      etos = etos_e1_e2_by_lm_lmInfluence_FALSE
    } else {
      etos = etos_e1_e2 
    }
    if(leverageOutput){
      h1=e1[,1]
      h2=h1
      etos = etos_e1_e2_by_lm
    }
  }
  
  if(is.null(partitionPrint))
    pPrint = TRUE
  else
    pPrint = partitionPrint
  
  if(nBig<=1)
    pPrint = FALSE
  
  if(pPrint)
    printi = capture.output(print(bigStrataLevels,row.names=FALSE))

  
  for(i in 1:nBig)
  {
    if(i==1 & pPrint){
      if(is.null(partitionPrint)){
        timeStart = Sys.time()
      } else {
        cat(printi[1],"\n")
      }
    }
    rows    = bigStrata  ==i
    lm_model = lm(c(calModel,as.formula(paste(response,"~1")))[[1]],data=grossSample[rows,])   # ~1 when calModel=NULL

    if(createPopTotals)
    {
      if(partitionPop|i==1)
      {
        rowsPop = bigStrataPop==i
        if(is.null(wPop)) wPopi = NULL
          else wPopi = wPop[rowsPop]
        tp = getTotal(popData[rowsPop,,drop=FALSE],lm_model,wPopi)
        popTotals_ =  tp$N*tp$colSum/tp$colN
      } else
        popTotals_ = NULL
    }
    else
    {
      popTotals_=setTotal(popTotalsInput[min(i,dim(popTotalsInput)[1]),,drop=TRUE],lm_model)
    }

    popTotals = rbind(popTotals,popTotals_)


    if(checkAnyNApopTotals){
      if(anyNA(popTotals[dim(popTotals)[1],])){
        if(!is.null(allowNApopTotals))
          stop("Missing population totals not allowed according to parameter allowNApopTotals.")
        else
          warning("Special methodology since missing population totals (warning according to parameter allowNApopTotals)")
        checkAnyNApopTotals = FALSE
      }
    }

    if(!onlyTotals & !(tolower(usePackage)=="nocalibration")){
      a = calibrateSSB(grossSample[rows,],calModel, popTotals[dim(popTotals)[1],],response=NULL,lRegModel,
                              popData=NULL,samplingW[rows],tolower(usePackage)=="survey",bounds=bounds,calfun= calfun, ##### "linear",
                              totalReturn=0,uselRegWeights=uselRegWeights,
                              ginvtol = ginvtol, boundstol = boundstol, ...)
      w[rows] = a$w
      if(wGrossOutput & !is.null(a$wGross))  wGross[rows] = a$wGross
    }
    if(residOutput){
      xFromModel = model.matrix(lm_model)
      rFromModel = model.frame(lm_model)[1]
      rowsNetto = rows
      rowsNetto[rowsNetto][!rFromModel==1] = FALSE
      e1_e2=etos(xFromModel[rFromModel==1,],
                       data.matrix(grossSample[rowsNetto,y]),
                       is.finite(popTotals[dim(popTotals)[1],]),
                       w=(samplingW[rowsNetto]),
                       ginvtol = ginvtol)
      e1[rowsNetto,]=e1_e2$e1
      e2[rowsNetto,]=e1_e2$e2
      if(leverageOutput){
        h1[rowsNetto]=e1_e2$h1
        h2[rowsNetto]=e1_e2$h2
      }
    }
    
    if(i==1 & pPrint & is.null(partitionPrint)){
      if(difftime(Sys.time(),timeStart,units = "secs")>timeLimit/nBig)
        cat(printi[1],"\n")
      else
        pPrint = FALSE
    }
    if(pPrint)
      cat(printi[1+i],"\n")
    

  }
  if(wGrossOutput) if(sum(is.finite(wGross))==0) wGross = NULL

  rownames(popTotals) =NULL
  popTotals = cbind(bigStrataLevels,popTotals)

  if(onlyTotals) return(popTotals)
  if(onlyw) return(w)


  ############## Almost same as in CalibratePackageReGenesees
  estTM=NULL
  if(!is.null(y)){
    if(is.list(y) | is.list(by)){
      if(is.list(y) & is.list(by)) {if(length(y)!=length(by)) stop("length(y)==length(by) must be TRUE")}
      else{
        if(is.list(y)){
          if(is.null(by))
            by = vector("list",length(y))
          else{
            by_ = by
            by = y
            for(i in 1:length(y)) by[[i]] = by_
          }
        }else{
          y_ = y
          y = by
          for(i in 1:length(y)) y[[i]] = y_
        }

      }
      estTM = y
      for(i in 1:length(y)) estTM[[i]] = MYestTM(grossSample,y[[i]],w,by[[i]])
    } else
    {
      estTM =  MYestTM(grossSample,y,w,by)
    }
  }

  dropResid2 = dropResid2  & (sum(is.na(popTotals))==0)

  retur=list(popTotals=popTotals,w=w)
  if(!is.null(estTM)) retur$estTM = estTM
  if(residOutput) {
    retur$resids=e1
    if(!dropResid2) retur$resids2=e2
    if(leverageOutput){
      retur$leverages=h1
      if(!dropResid2) retur$leverages2=h2
    }
  }
  if(yOutput) retur$y = grossSample[,y,drop=FALSE]
  if(samplingWeightsOutput) 
    retur$samplingWeights <- grossSample[, samplingWeights, drop = FALSE]
  if(wGrossOutput) retur$wGross = wGross
  } ######## END NOT ReGenesees
  
  if(!is.null(wave)){
    if(length(wave)>1)
      retur$wave = CrossStrata(grossSample[,wave])
    else
      retur$wave = grossSample[,wave, drop=TRUE]
  }
  if(!is.null(id)) retur$id = grossSample[,id, drop=TRUE]
  if(!is.null(extra)) retur$extra = grossSample[,extra,drop=FALSE]
  if(!residOutput | !yOutput)
    return(retur)
  return(structure(retur, class = "calSSB", n=n, nY=length(y)))
}



# Create new levels by crossing levels in "by"
# When returnb=TRUE an overview of original variabels according to new levels are also retuned
# byExtra contains the same variables as by and represents another data set.


#' Crossing several factor variables
#' 
#' Create new factor variable by crossing levels in several variables
#' 
#' 
#' @param by Dataframe or matrix with several variables
#' @param sep Used to create new level names
#' @param returnb When TRUE an overview of original variabels according to new
#' levels are also retuned.
#' @param asNumeric When TRUE the new variable is numeric.
#' @param byExtra Contains the same variables as by and represents another data
#' set.
#' @return \item{a}{The new variable} \item{aExtra}{New variable according to
#' byExtra} \item{b}{Overview of original variabels according to new levels}
#' @examples
#' 
#' CrossStrata(cbind(factor(rep(1:3,2)),c('A',rep('B',5)) ))
#' 
#' @export CrossStrata
CrossStrata = function(by,sep = "-",returnb=FALSE,asNumeric=FALSE,byExtra=NULL){
  by = as.data.frame(by)
  byList = as.list(by)
  b = sortrows(as.data.frame(aggregate(byList[[1]],byList,length)[,1:length(byList)]))
  rownames(b)=NULL
  names(b) = names(by)
  levels = apply(b,1,paste,collapse=sep)
  a      = factor(apply(by,1,paste,collapse=sep),levels = levels)
  if(asNumeric){
    levels_ = levels(a)
    a=as.numeric(a)
    attr(a,"levels") = levels_
  }
  if(!is.null(byExtra)) {
    aExtra = factor(apply(as.data.frame(byExtra),1,paste,collapse=sep),levels = levels)
    if(asNumeric){
      levels_ = levels(aExtra)
      aExtra=as.numeric(aExtra)
      attr(aExtra,"levels") = levels_
    }
    if(returnb) return(list(a=a,aExtra=aExtra,b=b))
    return(list(a=a,aExtra=aExtra))
  }
  if(returnb) return(list(a=a,b=b))
  a
}




#' Rearrange output from CalibrateSSB (calSSB object). Ready for input to PanelEstimation.
#' 
#' One row for each id and one column for each wave.
#' 
#' When wave, id or extra is NULL, corresponding elements in the input object (\code{a}) will be used if available,
#' 
#' 
#' @param a A calSSB object. That is, output from CalibrateSSB() or CalSSBobj().
#' @param wave Time or another repeat variable.
#' @param id Identifier variable.
#' @param subSet Grouping variable for splitting ouput.
#' @param extra Dataset with extra variables not in \code{a}.
#' @return Output has the same elements (+ extra) as input (a), but rearranged.
#' When subSet is input otput is alist according to the subSet levels.
#' 
#' @export
#' 
#' @seealso \code{\link{CalibrateSSB}}, \code{\link{CalSSBobj}}, \code{\link{PanelEstimation}}.
#' 
#' 
#' @examples
#' 
#' # See examples in PanelEstimation and CalSSBobj
#' 
WideFromCalibrate = function(a,wave=NULL,id=NULL,subSet=NULL,extra=NULL){    # wave instead of bigStrata, extra is list
  if(class(a)[1] != "calSSB")
    stop("a must be an object of class calSSB")
  if(is.null(wave))
    wave = a$wave
  if(is.null(wave))
    stop("wave neeed")
  if(is.null(id))
    id = a$id
  if(is.null(id))
    stop("id neeed")
  x=structure(list(),class = "calSSBwide")
  
  if(!is.null(a$y))          x$y          = wideDataMatrix(a$y,wave,id,asList=TRUE)
  if(!is.null(a$w))          x$w          = wideDataMatrix(a$w,wave,id,asList=FALSE)
  if(!is.null(a$resids))     x$resids     = wideDataMatrix(a$resids,wave,id,asList=TRUE)
  if(!is.null(a$resids2))    x$resids2    = wideDataMatrix(a$resids2,wave,id,asList=TRUE)
  if(!is.null(a$leverages))  x$leverages  = wideDataMatrix(a$leverages,wave,id,asList=FALSE)
  if(!is.null(a$leverages2)) x$leverages2 = wideDataMatrix(a$leverages2,wave,id,asList=FALSE)
  if(!is.null(a$samplingWeights)) x$samplingWeights = wideDataMatrix(a$samplingWeights,wave,id,asList=FALSE)
  if(!is.null(a$wGross))     x$wGross = wideDataMatrix(a$wGross,wave,id,asList=FALSE)
  if(!is.null(extra))        x$extra = wideDataMatrix(extra,wave,id,asList=TRUE)
  else
    if(!is.null(a$extra))      x$extra = wideDataMatrix(a$extra,wave,id,asList=TRUE)
  if(is.null(subSet)) return(x)
  s123 = make123(subSet)
  s = wideDataMatrix(s123,wave,id,asList=FALSE)
  k = vector("list",max(s123))
  k = structure(k,class = "calSSBwideList") # Ny her
  names(k) = levels(s123)
  for(i in 1:max(s123)){
    si = s==i
    rowsi = rowSums(si,na.rm=TRUE)>0
    k[[i]] = x
    for(j in 1:length(k[[i]]$y)) k[[i]]$y[[j]] = MakeSubSet(x$y[[j]],si,rowsi)
    for(j in 1:length(k[[i]]$resids)) k[[i]]$resids[[j]] = MakeSubSet(x$resids[[j]],si,rowsi)
    for(j in 1:length(k[[i]]$resids2)) k[[i]]$resids2[[j]] = MakeSubSet(x$resids2[[j]],si,rowsi)
    for(j in 1:length(k[[i]]$extra)) k[[i]]$extra[[j]] = MakeSubSet(x$extra[[j]],si,rowsi)
    k[[i]]$w = MakeSubSet(x$w,si,rowsi)
    k[[i]]$leverages = MakeSubSet(x$leverages,si,rowsi)
    k[[i]]$leverages2 = MakeSubSet(x$leverages2,si,rowsi)
    k[[i]]$samplingWeights = MakeSubSet(x$samplingWeights,si,rowsi)
    k[[i]]$wGross = MakeSubSet(x$wGross,si,rowsi)
  }
  k
}

# Old name bigData
# Can be simplified
wideDataMatrix = function(data,bigStrata=rep(1,n),id,nameSep="-",dropSingleName=TRUE,asList=FALSE)
{
  n <- NROW(data)
  if(!is.matrix(data)) data = data.matrix(data)
  if(!is.null(colnames(data))) varNames = colnames(data)
    else varNames = paste("y",1:dim(data)[2],sep="")
  if(asList){
    k = vector("list",dim(data)[2])
    names(k) = varNames
    for(i in 1:dim(data)[2])
      k[[i]] = wideDataMatrix(data=data[,i,drop=FALSE],bigStrata=bigStrata,id=id,dropSingleName=TRUE)
    return(k)
  }
  dropName = (dim(data)[2]==1 & dropSingleName)
  if(dropName) nameSep=""
  bigStrata = make123(bigStrata)
  id=make123(id)
  nBig    = max(bigStrata)
  bigNames  = paste(nameSep,levels(bigStrata),sep="")
  x0 = matrix(NaN,nrow=max(id),ncol=dim(data)[2])
  z=NULL
  for(i in 1:nBig)
  {
    x=x0
    datai = data[bigStrata==i,,drop=FALSE]
    x[id[bigStrata==i],] = datai
    if(dropName) colnames(x) = bigNames[i]
      else colnames(x) = paste(varNames,bigNames[i],sep="")
    z=cbind(z,x)
  }
  z
}


make123 = function(x)
{
  x=as.factor(x)
  levels_x = levels(x)
  x=as.numeric(x)
  attr(x,"levels") = levels_x
  x
}

rBind = function(x,y) rbind(data.frame(x),data.frame(y))[[1]]


uniqueIndex = function(x,useRev=FALSE){
  if(useRev) x = rev(x)
  ix = unique(match(x,x))
  if(!useRev) return(ix)
  sort((length(x):1)[ix])
}

uniqueCol= function(x,useRev=FALSE){
  if(is.null(dim(x))) {
    namesx=names(x)
    x=matrix(x,nrow=1)
    colnames(x)=namesx
  }
  x[,uniqueIndex(colnames(x),useRev),drop=FALSE]
}

sortrows <- function(m, cols = 1:dim(m)[2], index.return = FALSE) {
  ix <- eval(parse(text = paste("order(", paste("m[[", cols, "]]", sep = "", collapse = ","),  ")")))
  if (index.return) 
    return(ix)
  m[ix, , drop = FALSE]
}

sortrowsOld = function(m,cols=1:dim(m)[2],index.return=FALSE)
{
  ix=eval(parse(text=paste("order(",paste("m[,",cols,"]",sep="", collapse=","),")")))
  if(index.return) return(ix)
  m[ix, ,drop=FALSE]
}


MYestTMold = function(grossSample,y,w,by){
  if(!is.null(by)) if(is.na(by[1])) by = NULL
  estTM =aggregate(grossSample[,y]*w,grossSample[,by,drop=FALSE],sum_)
  if(!is.null(by)) estTM = sortrows(estTM,1:length(by))
  if(length(y)==1) names(estTM)[length(names(estTM))] = y
  estTM
}

MYestTM = function(grossSample,y,w,by){
  if(!is.null(by)) if(is.na(by[1])) by = NULL
  #grossSample <<- grossSample
  #w <<- w
  #y <<- y
  #estTM =aggregate(grossSample[,y, drop = TRUE]*w,grossSample[,by,drop=FALSE],sum_)
  
  
  if(!is.null(by)){
    #cat("#")
    estTM = sortrows(aggregate(grossSample[,y, drop = TRUE]*w,grossSample[,by,drop=FALSE],sum_),1:length(by))
  } else {
    #cat("0")
    estTM =aggregate(grossSample[,y, drop = TRUE]*w,grossSample[,integer(0),drop=FALSE],sum_)
  }
  
  if(length(y)==1) names(estTM)[length(names(estTM))] = y
  estTM
}



mergeSort = function(x,y,useRev=FALSE,remov=FALSE){
  x=as.data.frame(x)
  y=as.data.frame(y)
  if(NROW(x)!=NROW(y)) stop("Not exact match")
  colnamesx   = colnames(x)
  colnames(x) = paste(colnamesx,"_",sep="")
  uniqueIndex_ = uniqueIndex(colnamesx,useRev)
  colnames(x)[uniqueIndex_] = colnamesx[uniqueIndex_]
  mergeSortNr123 = 1:NROW(x)
  mergeSortNr345 = 1:NROW(x)
  names_y=names(y)
  z=merge(cbind(x,mergeSortNr345),cbind(y,mergeSortNr123),names_y)
  if(NROW(x)!=NROW(z)) stop("Not exact match")
  k=dim(z)[2]
  if(remov) cols = !(colnames(x) %in% names_y)
  else cols = rep(TRUE,length(colnamesx))
  colnames(x) = colnamesx
  x[z[order(z[,"mergeSortNr123"]),"mergeSortNr345"],cols]
}

MakeSubSet = function(x,subSet,rows){
  if(is.null(x)) return(x)
  x[!subSet] = NaN
  x[rows,,drop=FALSE]
}


sum_ = function(x) sum(x,na.rm=TRUE)



asFormula = function(s) {
  if(is.null(s)) return(NULL)
  if(class(s)[1]=="formula") return(s)
  if(is.na(s)[1]) return(NULL)
  as.formula(paste("~",paste(s,collapse="+"),sep=""))
}


CalibratePackageReGeneseesCRANversion = function(netSample,calmodel=NULL,popTotals=NULL,y=NULL,by = NULL,partition=NULL,
                          popData=NULL,samplingWeights=NULL,bounds=c(-Inf,Inf),calfun="linear",
                          onlyTotals=FALSE,ids,...){
  stop("Use of ReGenesees is not implemented in this version since ReGenesees is not on CRAN.")
}





calibrateSSB = function(grossSample,calModel=NULL,
                        popTotals=NULL,response=NULL,lRegModel=NULL,
                        popData=NULL,samplingWeights=NULL,
                        usePackageSurvey=TRUE,bounds=c(-Inf,Inf),calfun="linear",
                        totalReturn=0,uselRegWeights=FALSE, ### Merk: uselRegWeights=FALSE
                        ginvtol, boundstol, ...)         
{
  n = NROW(grossSample)

  w=samplingWeights

  if(uselRegWeights) lRegWeights=samplingWeights
  else lRegWeights=NULL


  if(!is.null(lRegModel))
  {
    if(is.null(lRegWeights)) glm_modell = glm(as.formula(lRegModel),data=grossSample,family = binomial())
    else glm_modell = glm(as.formula(lRegModel),data=grossSample,family = binomial(),weights=lRegWeights)
    lRegW = 1/predict(glm_modell,type = "response")
    lRegW[model.frame(glm_modell)[1]==0] = 0
    if(is.null(w)) w=1
    w = w*lRegW

    if(length(popTotals)>0)
    {
      w = w*popTotals[1]/sum(w)
    }
  }
  a = list(w=w,wGross=NULL)

  if(!is.null(calModel))
  {
    if(usePackageSurvey)
    {
      a=calibratePackageSurvey(grossSample=grossSample,modelformula=calModel,
                               popTotals=popTotals,response=response,popData=popData,samplingWeights=w,
                               bounds=bounds,calfun=calfun,totalReturn=totalReturn,returnwGross=TRUE,...)
    } else


    {
      if(!(calfun=="linear")) stop("Method not implemented")
      if(!is.null(popTotals)) popData=popTotals
      a=lagVekter(calModel,grossSample,popData,min_w = bounds[1],
                  max_w = bounds[2],totalReturn=totalReturn,samplingWeights=w,returnwGross=TRUE,
                  ginvtol = ginvtol, boundstol = boundstol)
    }
  }
  a
}

calibratePackageSurvey = function(grossSample,modelformula,popTotals=NULL,response=NULL,
          popData=NULL,samplingWeights=NULL,calfun="linear",totalReturn=0,returnwGross=FALSE,...)
{
  if(returnwGross) calWeights1 = NULL
  modelformula =as.formula(modelformula)

  lm_model = lm(modelformula,data=grossSample)
  xFromModel = model.matrix(lm_model)
  rFromModel = model.frame(lm_model)[1]

  if(is.null(popTotals))
  {
    if(is.null(popData)) tp = getTotal(grossSample,lm_model,samplingWeights)   # Ny
	else tp = getTotal(popData,lm_model)

	if(totalReturn==2) return(tp)
    popTotals =  tp$N*tp$colSum/tp$colN
  }
  else
  {
    popTotals=setTotal(popTotals,lm_model)
  }

  if(totalReturn==1) return(popTotals)

  if(sum(is.na(popTotals))>0)
  {
    # not netDesign but same name
    netDesign = svydesign(ids=~1,data=data.frame(rFromModel,xFromModel),weights=samplingWeights)
    col1= !is.na(popTotals)
    col1[1] = FALSE     # Antar Intercept nr 1 og denne tas bort
    varNames1 = colnames(xFromModel)[col1] # Variabler der totaler finnes
    modelformula1 = update(modelformula,paste("~ ",paste(varNames1,collapse="+")))
    calWeights1 = try(weights(calibrate(netDesign,modelformula1 ,popTotals[!is.na(popTotals)],calfun=calfun,...)),silent=FALSE)
    if(class(calWeights1)[1]=="try-error")
      stop ("The function calibrate in package survey did not succeed. Maybe change the parameter usePackage?")
    
    
    popTotals[is.na(popTotals)] =  colSums(calWeights1*xFromModel[,is.na(popTotals)])
  }

  if(!(totalReturn==0)) return(popTotals)

  netDesign = svydesign(ids=~1,data=grossSample[rFromModel==1,],weights=samplingWeights[rFromModel==1])

  calWeights = rep(0,dim(grossSample)[1])
  survey_weights = try(weights(calibrate(netDesign,modelformula ,popTotals,calfun=calfun,...)),silent=FALSE)
  
  if(class(survey_weights)[1]=="try-error")
    stop ("The function calibrate in package survey did not succeed. Maybe change the parameter usePackage?")
  
  calWeights[rFromModel==1] = survey_weights 
  
  if(returnwGross){
    return(list(w=calWeights,wGross=calWeights1))
  }
  calWeights
}

# (5.3.14) in ETOS manual (without "- -" typing error)
# (11.8) in Sarndal-Lundstrom-2005
# Without summing over strata
etos_5.3.14 = function(N,n,g,e)
{
  ge=g*e
  (N^2*(1-n/N)/(n*(n-1))) * (sum(ge^2) - (sum(ge))^2/n ) - (N/n)*(N/n-1)*sum(ge*(g-1)*e)
}


# (5.3.15) in ETOS manual
# (11.9) in Sarndal-Lundstrom-2005
# Without summing over strata
etos_5.3.15 = function(N,n,g,e)
{
  (N/n)^2*sum(g*(g-1)*e^2)
}


# Beregner residualer fra modelmatrise og matrise med y-er
# isTotPop er vektor som sier hvilke variabler det finnes tot-pop for
#          Det legges til FALSE om vektoren er for kort
#          Default er at bare N er kjent i populasjonen
# w er designvekter
etos_e1_e2 = function(mNetto,yMatrix,isTotPop=TRUE,ginvtol,w=NULL)
{
  inv = function(x) my_ginv(x,tol=ginvtol)
  if(is.null(w)) w = 1
  sqrt_w = sqrt(w)

  isTotPop_input = isTotPop
  isTotPop = rep(FALSE,dim(mNetto)[2])
  for(i in 1:length(isTotPop_input))
    isTotPop[i] = isTotPop_input[i]

  reswreg = function(y,x,sqrt_w)
    y-(x %*% inv(x*sqrt_w)*sqrt_w) %*% y
  a=NULL
  a$e1=reswreg(yMatrix,mNetto[,isTotPop,drop=FALSE],sqrt(w))
  if(sum(!isTotPop)==0) a$e2=a$e1
  else a$e2 = reswreg(yMatrix,mNetto,sqrt(w))
  a
}


etos_e1_e2_by_lm = function(mNetto,yMatrix,isTotPop=TRUE,lmInfluence=TRUE,w=NULL, ginvtol) # ginvtol not used 
{

  isTotPop_input = isTotPop
  isTotPop = rep(FALSE,dim(mNetto)[2])
  for(i in 1:length(isTotPop_input))
    isTotPop[i] = isTotPop_input[i]

  a=NULL
  m   = lm(yMatrix~mNetto,weights=w)
  a$e1 = resid(m)
  a$e2 = a$e1
  if(lmInfluence){
    a$h1 = lm.influence(m)$hat
    a$h2 = a$h1
  }
  if(!(sum(!isTotPop)==0)){
    mNetto = mNetto[,isTotPop,drop=FALSE]
    m   = lm(yMatrix~mNetto,weights=w)
    a$e1 = resid(m)
    if(lmInfluence) a$h1 = lm.influence(m)$hat
  }
  a
}

etos_e1_e2_by_lm_lmInfluence_FALSE <- function(..., lmInfluence = FALSE) {
  etos_e1_e2_by_lm(..., lmInfluence = lmInfluence)
}


etosV1V2 = function(e1,e2,w,samplingWeights=NULL,R=is.finite(e1),  N = sum(w,na.rm=T),n = sum(is.finite(w))){

  if(is.null(samplingWeights))   g=w*(n/N)
  else g = w/samplingWeights

  V1 = etos_5.3.14(N,n,g[R],e1[R])
  V2 = etos_5.3.15(N,n,g[R],e2[R])

  list(V1=V1,V2=V2)
}

ImpVekt = function(mBrutto,r,
            totPop=dim(mBrutto)[1],N_brutto=TRUE,totPopReturn=FALSE,singularReturn=FALSE,ginvtol, w=NULL)
{
  wGross = NULL
  if(singularReturn) totPopReturn=TRUE
       inv = function(x) my_ginv(x,tol=ginvtol)

  if(is.null(w)) w = 1
  sqrt_w = sqrt(w)


  totPop_input = totPop
  totPop = rep(NaN,dim(mBrutto)[2])
  for(i in 1:length(totPop_input))
    totPop[i] = totPop_input[i]
  totPop_input = totPop

  tp = !is.na(totPop)

  wregw = function(tot,x,sqrt_w)
  {
     (tot %*% inv(x*sqrt_w)*sqrt_w)[,]
  }


  if(sum(!tp)>0)
     wGross = wregw(totPop[tp],mBrutto[,tp,drop=FALSE],sqrt_w)
     totPop[!tp] = colSums(mBrutto[,!tp,drop=FALSE]*wGross)

  if(length(sqrt_w)>1)	sqrt_w=sqrt_w[r==1]
  vekt = wregw(totPop,mBrutto[r==1,,drop=FALSE],sqrt_w)


  if(N_brutto)
  {
    vekt_netto = vekt
    vekt = r
    vekt[r==1] = vekt_netto
  }


  vekt = as.vector(unlist(vekt,use.names=FALSE))
  if(totPopReturn)
  {
    totP        = totPop
	  totPop      = totPop_input
    if(singularReturn)
      return(list(vekt=vekt,totPop=totPop,wGross=wGross))
      #singular1=svd(R,nv=0,nu=0)$d, # R gir samme svar som mBrutto
      #singular2=svd(qtw %*%q,nv=0,nu=0)$d))
    return(list(vekt=vekt,totPop=totPop,wGross=wGross))
  }
  vekt
}





glmR2ImpVekt = function(glmR,...)
{
   ImpVekt(model.matrix(glmR),glmR$y==1,...)
}


ImpVektFixed = function(mNetto,totPop,wFixed=NULL,useginv=TRUE,ginvtol)
{

  if(useginv)
       inv = function(x) my_ginv(x,tol=ginvtol)
  else inv = function(x) solve(x)

  qr_mNetto = qr(mNetto) # mNetto = Q*R
  Q = qr.Q(qr_mNetto)
  R = qr.R(qr_mNetto)
  invR = inv(R)

  totPop_invR = totPop %*% invR

  vekt = as.vector(totPop_invR %*% t(Q))

  if(is.null(wFixed)) return(vekt)

  z = matrix(wFixed - vekt,length(vekt),1)


  za = z[!is.na(wFixed),,drop=FALSE]
  Qa = Q[!is.na(wFixed),,drop=FALSE]
  Qb = Q[is.na(wFixed),,drop=FALSE]


  d =  - t(Qa)  %*% za


  zb = (Qb %*% inv(t(Qb)%*%Qb)) %*% d


  z[is.na(z)] = zb

  nyVekt = as.vector(z) + vekt

  return(nyVekt)
}

#' Print method for calSSB
#'
#' @param x calSSB object 
#' @param digits positive integer.  Minimum number of significant digits to be used for printing most numbers.
#' @param \dots further arguments sent to the underlying
#'
#' @return Invisibly returns the original object.
#' @keywords print
#' @export
print.calSSB <- function(x, digits = max(getOption("digits") - 3, 3), ...) {
  if(!is.null(x$estTM)){
    cat("---- estTM ----\n")
    print(x$estTM)
    cat("\n\n")
  }  
  
  cat("---- n =", attr(x,"n"),"    ny =",attr(x,"nY"),"\n\n")
  
  if(!is.null(colnames(x$y)))
    cat("---- y names ----\n",head(colnames(x$y),21),"\n\n")
  
  cat("---- Calibrated weights, w ----\n",head(x$w),"...",tail(x$w,1),"\n\n")
  
  cat("---- Summary of calSSB object ----\n")
  print(summary(x,digits = digits,...))
  invisible(x)
}


#' Print method for calSSBwide
#'
#' @param x calSSBwide object  
#' @param digits positive integer.  Minimum number of significant digits to be used for printing most numbers.
#' @param \dots further arguments sent to the underlying
#'
#' @return Invisibly returns the original object.
#' @keywords print
#' @export
print.calSSBwide <- function(x, digits = max(getOption("digits") - 3, 3), ...) {
  
  cat("---- ",NCOL(x$w)," waves of ",NROW(x$w)," units ---- \n\n")
  
  if(!is.null(colnames(x$w)))
    cat("---- wave names ----\n",head(colnames(x$w),21),"\n")
  
  cat("\n---- Summary of calSSBwide object ----\n")
  print(summary(x,digits = digits,...))
  cat("\n---- Summary of y ----\n")
  print(summary(x$y,digits = digits,...))
  if(!is.null(x$extra)){
    cat("\n---- Summary of extra ----\n")
    print(summary(x$extra,digits = digits,...))
  }
  
  invisible(x)
}




CheckDim <- function(x,nRow,nCol=1,nam="x",allowNULL=TRUE){
  if(allowNULL & is.null(x)) return(NULL)
  if(NROW(x) !=nRow | NCOL(x) !=nCol)
    stop(paste("Wrong dimension of",nam))
  NULL
}



#' Create or modify a CalSSB object
#' 
#' The elements of the CalSSB object are taken directly from the input parameters.
#'
#' @param x NULL or an  existing calSSB object
#' @param y y
#' @param w w
#' @param wGross wGross
#' @param resids resids
#' @param resids2 resids2
#' @param leverages leverages
#' @param leverages2 leverages2
#' @param samplingWeights samplingWeights
#' @param extra extra
#' @param id id 
#' @param wave wave 
#'
#' @return A CalSSB object. That is, an object of the type retuned by \code{\link{CalibrateSSB}}.
#' 
#' @note If x is a ReGenesees/cal.analytic object, this function is a wrapper to \code{\link{CalSSBobjReGenesees}}.
#' 
#' @export
#' 
#' @seealso \code{\link{CalibrateSSB}}, \code{\link{CalSSBobjReGenesees}}, \code{\link{WideFromCalibrate}},  \code{\link{PanelEstimation}}.
#'
#' @examples
#' #' # Generates data - two years
#' z <- AkuData(3000)  # 3000 in each quarter
#' zPop <- AkuData(10000)[, 1:7]
#' 
#' # Create a CalSSB object by CalibrateSSB
#' b <- CalibrateSSB(z, calmodel = "~ sex*age", partition = c("year", "q"), popData = zPop, 
#'                   y = c("unemployed", "workforce"))
#' 
#' # Modify the CalSSB object
#' a <- CalSSBobj(b, w = 10*b$w, wave = CrossStrata(z[, c("year", "q")]), id = z$id)
#' 
#' # Use the CalSSB object as input ...
#' PanelEstimation(WideFromCalibrate(a), "unemployed", linComb = PeriodDiff(8, 4))
#' 
#' # Create CalSSB object without x as input
#' CalSSBobj(y = b$y, w = 10*b$w, resids = b$resids, wave = CrossStrata(z[, c("year", "q")]), 
#'           id = z$id)
#' 
CalSSBobj <- function(x=NULL,y=NULL,w=NULL,wGross=NULL,resids=NULL,resids2=NULL,
                   leverages=NULL,leverages2=NULL,samplingWeights=NULL,extra=NULL,
                   id=NULL, wave=NULL){
  if(!is.null(x)){
    if(class(x)[1]=="cal.analytic"){
      if(!is.null(w))         warning("Input w ignored when ReGenesees")
      if(!is.null(wGross))    warning("Input wGross ignored when ReGenesees")
      if(!is.null(resids))    warning("Input resids ignored when ReGenesees")
      if(!is.null(resids2))   warning("Input resids2 ignored when ReGenesees")
      if(!is.null(leverages)) warning("Input leverages ignored when ReGenesees")
      if(!is.null(leverages)) warning("Input leverages2 ignored when ReGenesees")
      return(CalSSBobjReGenesees(x,
        y=y,samplingWeights=samplingWeights,extra=extra,id=id,wave=wave))
    }
    if(class(x)[1] != "calSSB")
      stop("x must be an object of class calSSB")
    n = attr(x,"n")
    nY = attr(x,"nY")
    CheckDim(y,n,1,"y")
    if(!is.null(y)) x$y = y
  }
  else{
    if(is.null(y)) stop("y needed in input when x=NULL")
    if(is.null(w)) stop("w needed in input when x=NULL")
    if(is.null(resids)) stop("resids needed in input when x=NULL")
    n = dim(y)[1]
    nY= dim(y)[2]
    x = structure(list(y=y), class = "calSSB", n=n, nY=nY)
  }
  CheckDim(w,n,1,"w")
  CheckDim(wGross,n,1,"wGross")
  CheckDim(resids,n,nY,"resids")
  CheckDim(resids2,n,nY,"resids2")
  CheckDim(leverages,n,1,"leverages")
  CheckDim(leverages2,n,1,"leverages2")
  CheckDim(samplingWeights,n,1,"samplingWeights")
  if(!is.null(extra))
    if(NROW(extra) != n) 
      stop(paste("extra must have ",n," rows"))
  CheckDim(id,n,1,"id")
  if(!NCOL(wave)==1)
    wave = CrossStrata(wave)
  CheckDim(wave,n,1,"wave")
  if(!is.null(w)) x$w = w
  if(!is.null(wGross)) x$wGross = wGross
  if(!is.null(resids)) x$resids = resids
  if(!is.null(resids2)) x$resids2 = resids2
  if(!is.null(leverages)) x$leverages = leverages
  if(!is.null(leverages2)) x$leverages2 = leverages2
  if(!is.null(samplingWeights)) x$samplingWeights = samplingWeights
  if(!is.null(extra)) x$extra = extra
  if(!is.null(id)) x$id = id
  if(!is.null(wave)) x$wave = wave
  x
}


#' Create a CalSSB object from a ReGenesees/cal.analytic object 
#'
#' @param x Output from ReGenesees::e.calibrate() (object of class cal.analytic)  
#' @param y formula or variable names 
#' @param samplingWeights NULL, TRUE (capture from x), formula, variable name or vector of data
#' @param extra NULL, formula, variable names or matrix of data
#' @param id NULL, TRUE (ids from x), formula, variable name or vector of data
#' @param wave NULL,  formula, variable name or vector of data
#'
#' @return A CalSSB object. That is, an object of the type retuned by \code{\link{CalibrateSSB}}.
#' @export
#' 
#' @seealso \code{\link{CalibrateSSB}}, \code{\link{CalSSBobj}}, \code{\link{WideFromCalibrate}},  \code{\link{PanelEstimation}}.
#' 
#' @examples
#' \dontrun{
#' # Generates data - two years
#' z <- AkuData(3000)  # 3000 in each quarter
#' zPop <- AkuData(10000)[, 1:7]
#' z$samplingWeights <- 1
#' z$ids <- 1:NROW(z)
#' 
#' # Create a ReGenesees/cal.analytic object
#' library("ReGenesees")
#' desReGenesees <- e.svydesign(z[z$R == 1, ], ids = ~ids, weights = ~samplingWeights)
#' popTemplate <- pop.template(data = desReGenesees, calmodel = ~sex * age, partition = ~year + q)
#' popTotals <- fill.template(universe = zPop, template = popTemplate)
#' calReGenesees <- e.calibrate(design = desReGenesees, df.population = popTotals)
#' 
#' # Create CalSSB objects from a ReGenesees/cal.analytic object
#' CalSSBobjReGenesees(calReGenesees, y = ~unemployed + workforce, id = TRUE, 
#'                     samplingWeights = TRUE, extra = ~famid)
#' a <- CalSSBobjReGenesees(calReGenesees, y = c("unemployed", "workforce"), 
#'                          id = "id", extra = "famid", wave = c("year", "q"))
#' 
#' # Use the CalSSB object as input ...
#' PanelEstimation(WideFromCalibrate(a), "unemployed", linComb = PeriodDiff(8, 4))
#' 
#' }
CalSSBobjReGenesees <- function(x,y, samplingWeights=NULL,extra=NULL,
                      id=NULL, wave=NULL){
  if (requireNamespace("ReGenesees", quietly = TRUE)) {
    get.residuals <- ReGenesees::get.residuals
  } else {
    stop("The package ReGenesees, is needed.")
  }
  z = NULL
  z$w <- weights(x)
  z$resids = get.residuals(x,asFormula(y), scale = "no")
  z$y = model.frame(asFormula(y), data=x$variables)
  n = dim(z$y)[1]
  nY= dim(z$y)[2]
  
  if(!is.null(samplingWeights)){
    if(is.logical(samplingWeights)){
      if(samplingWeights){
        wei      <- attr(x, "weights")
        wei.char <- all.vars(wei)
        samplingWeights <- substr(wei.char, 0, nchar(wei.char) - 4)
        } else
          samplingWeights <- NULL
        } 
  }
  if(!is.null(samplingWeights))
    if(class(samplingWeights)[1]=="formula" | (is.character(samplingWeights) & length(samplingWeights)==1 ))
      samplingWeights = model.frame(asFormula(samplingWeights), data=x$variables)[,]
  z$samplingWeights = samplingWeights
  
  if(!is.null(extra))
    if(class(extra)[1]=="formula" | (is.character(extra) & length(extra)<min(dim(x$variables)) ))
      extra = model.frame(asFormula(extra), data=x$variables)    
  z$extra = extra
    
  if(!is.null(id))
    if(is.logical(id)){
       if(id)
         id = model.frame(attr(x,"ids"),data=x$variables)[,]
       else
         id = NULL
    }
  
  if(!is.null(id))
    if(class(id)[1]=="formula" | (is.character(id) & length(id)==1))
      id = model.frame(asFormula(id), data=x$variables)[,]    
  z$id = id
  
  if(!is.null(wave))
      if(class(wave)[1]=="formula" | (is.character(wave) & length(wave)<min(dim(x$variables)) ))
        wave = model.frame(asFormula(wave), data=x$variables)[,]
  if(NCOL(wave)==1)
    z$wave = wave
  else
    z$wave = CrossStrata(wave)
    
    
  n = dim(z$y)[1]
  nY= dim(z$y)[2]
  structure(z, class = "calSSB", n=n, nY=nY)
}





GetSubset <- function(x, subset) {
  if (is.null(x))
    return(x)
  if (is.null(subset))
    return(x)
  if(is.null(ncol(x))) x=as.vector(x) ## Handle problem with AsIs-class in special cases
  if (is.vector(x))
    return(x[subset])
  return(x[subset, , drop = FALSE])
}


RemoveZeroCalSSB <- function(x){
    s = x$w!=0    
    for(i in 1: length(x))
      x[[i]] = GetSubset(x[[i]],s)  ## Unngå å gjøre dette for popTotals, estTM
    x    ################ Må endre nY atributt også
}
  
  



#' Calibration weighting and variance estimation for panel data
#' 
#' @encoding UTF8
#'
#' @param ... Input to CalibrateSSB() and PanelEstimation()
#'
#' @return Output from PanelEstimation()
#' @export
#' 
#' @seealso \code{\link{CalibrateSSB}}, \code{\link{PanelEstimation}}.
#'
#' @examples
#' z    = AkuData(3000)  # 3000 in each quarter
#' zPop = AkuData(10000)[,1:7]
#' lc = rbind(LagDiff(8,4),PeriodDiff(8,4))
#' rownames(lc) = c("diffQ1","diffQ2","diffQ3","diffQ4","diffYearMean")
#' CalibrateSSBpanel(grossSample=z,calmodel="~ sex*age", partition=c("year","q"),popData=zPop, 
#'        y=c("unemployed","workforce"),id="id",wave=c("year","q"),
#'        numerator="unemployed",linComb=lc)
CalibrateSSBpanel = function(...){
  sysCall <- sys.call()
  panelArgs <-  names(sysCall) %in% formalArgs(PanelEstimation)
  CalibrateArgs <- !panelArgs 
  sysCallPanel <- as.call(c(list(as.name("PanelEstimation"),x="x"),as.list(sysCall[panelArgs])) )
  sysCall <- sysCall[!panelArgs]  
  sysCall[[1]] <- as.name("CalibrateSSB")
  parentFrame = parent.frame()
  sysCallPanel$x = WideFromCalibrate(eval(sysCall, envir=parentFrame))
  eval(sysCallPanel, envir=parentFrame)
}






#bB=RemoveZeroCalSSB(CalSSBobj(b,id=z$id))
#bWideB = WideFromCalibrate(bB,CrossStrata(z[b$w!=0 ,c("year","q")]))

#' OrderedVarNames
#'
#' @param x input 
#' @param sep character string used to separate
#'
#' @return output
#' @keywords internal
#' @export
#'
#' @examples
#' z <- data.frame(A = factor(c("a", "b", "c")), B = factor(1:2), C = 1:6)
#' x <- colnames(model.matrix(~B * C * A, z))
#' OrderedVarNames(x)
OrderedVarNames <- function(x, sep = ":") {
  unlist(lapply(strsplit(x, sep), function(x) paste(sort(x), collapse = sep)))
}


#' MatchVarNames
#'
#' @param x x
#' @param y y
#' @param sep sep 
#' @param makeWarning Warning when matching by reordering
#'
#' @return An integer vector giving the position in y of the first match if there is a match, otherwise NA.
#' @keywords internal
#' @export
#'
#' @examples
#' z <- data.frame(A = factor(c("a", "b", "c")), B = factor(1:2), C = 1:6)
#' x <- colnames(model.matrix(~B * C * A, z))
#' y <- colnames(model.matrix(~A * B + A:B:C, z))
#' MatchVarNames(x, y)
MatchVarNames <- function(x, y, sep = ":", makeWarning = FALSE) {
  matchNames <- match(x, y)
  noMatch <- is.na(matchNames)
  if (!any(noMatch)) 
    return(matchNames)
  matchNames[noMatch] <- match(OrderedVarNames(x[noMatch]), OrderedVarNames(y))
  if (makeWarning) 
    if (any(!is.na(matchNames[noMatch]))) 
      warning("Matching by reordering")
  matchNames
}

setTotal <- function(total, lmObject) {
  x <- model.matrix(lmObject)[1, ]
  x[] <- NA
  matchNames <- MatchVarNames(names(x), names(total), makeWarning = FALSE)
  if (any(duplicated(matchNames[!is.na(matchNames)]))) 
    stop("Duplicates when matching")
  x[] <- total[matchNames]
  x
}

getTotal <- function(data, lmObject, w = NULL) {
  mfCal <- model.frame(lmObject)
  mfCalNA <- mfCal[c(NA, 1), ][1, , drop = FALSE]
  rownames(mfCalNA) <- NULL
  mfCalNames <- colnames(mfCal)
  okNames <- mfCalNames %in% colnames(data)
  if (any(!okNames)) 
    data <- cbind(data[, mfCalNames[okNames], drop = FALSE], mfCalNA[, mfCalNames[!okNames], drop = FALSE])
  m <- model.matrix(lmObject, data = data, na.action = NULL)
  
  x <- NULL
  if (is.null(w)) {
    x$colSum <- colSums(m, na.rm = TRUE)
    x$N <- dim(m)[1]
    if (anyNA(m)) {
      x$colN <- colSums(!is.na(m))
    } else {
      x$colN <- rep(x$N, length(x$colSum))
      names(x$colN) <- names(x$colSum)
    }
  } else {
    x$colSum <- colSums(w * m, na.rm = TRUE)
    x$N <- sum(w)
    if (anyNA(m)) {
      # will not happen
      x$colN <- colSums(w * (!is.na(m)))
    } else {
      x$colN <- rep(x$N, length(x$colSum))
      names(x$colN) <- names(x$colSum)
    }
  }

  x
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CalibrateSSB.R
\name{WideFromCalibrate}
\alias{WideFromCalibrate}
\title{Rearrange output from CalibrateSSB (calSSB object). Ready for input to PanelEstimation.}
\usage{
WideFromCalibrate(a, wave = NULL, id = NULL, subSet = NULL, extra = NULL)
}
\arguments{
\item{a}{A calSSB object. That is, output from CalibrateSSB() or CalSSBobj().}

\item{wave}{Time or another repeat variable.}

\item{id}{Identifier variable.}

\item{subSet}{Grouping variable for splitting ouput.}

\item{extra}{Dataset with extra variables not in \code{a}.}
}
\value{
Output has the same elements (+ extra) as input (a), but rearranged.
When subSet is input otput is alist according to the subSet levels.
}
\description{
One row for each id and one column for each wave.
}
\details{
When wave, id or extra is NULL, corresponding elements in the input object (\code{a}) will be used if available,
}
\examples{

# See examples in PanelEstimation and CalSSBobj

}
\seealso{
\code{\link{CalibrateSSB}}, \code{\link{CalSSBobj}}, \code{\link{PanelEstimation}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setAndGetTotal.R
\name{MatchVarNames}
\alias{MatchVarNames}
\title{MatchVarNames}
\usage{
MatchVarNames(x, y, sep = ":", makeWarning = FALSE)
}
\arguments{
\item{x}{x}

\item{y}{y}

\item{sep}{sep}

\item{makeWarning}{Warning when matching by reordering}
}
\value{
An integer vector giving the position in y of the first match if there is a match, otherwise NA.
}
\description{
MatchVarNames
}
\examples{
z <- data.frame(A = factor(c("a", "b", "c")), B = factor(1:2), C = 1:6)
x <- colnames(model.matrix(~B * C * A, z))
y <- colnames(model.matrix(~A * B + A:B:C, z))
MatchVarNames(x, y)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CalibrateSSB.R
\name{CrossStrata}
\alias{CrossStrata}
\title{Crossing several factor variables}
\usage{
CrossStrata(by, sep = "-", returnb = FALSE, asNumeric = FALSE, byExtra = NULL)
}
\arguments{
\item{by}{Dataframe or matrix with several variables}

\item{sep}{Used to create new level names}

\item{returnb}{When TRUE an overview of original variabels according to new
levels are also retuned.}

\item{asNumeric}{When TRUE the new variable is numeric.}

\item{byExtra}{Contains the same variables as by and represents another data
set.}
}
\value{
\item{a}{The new variable} \item{aExtra}{New variable according to
byExtra} \item{b}{Overview of original variabels according to new levels}
}
\description{
Create new factor variable by crossing levels in several variables
}
\examples{

CrossStrata(cbind(factor(rep(1:3,2)),c('A',rep('B',5)) ))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calSSB.R
\encoding{UTF8}
\name{CalibrateSSBpanel}
\alias{CalibrateSSBpanel}
\title{Calibration weighting and variance estimation for panel data}
\usage{
CalibrateSSBpanel(...)
}
\arguments{
\item{...}{Input to CalibrateSSB() and PanelEstimation()}
}
\value{
Output from PanelEstimation()
}
\description{
Calibration weighting and variance estimation for panel data
}
\examples{
z    = AkuData(3000)  # 3000 in each quarter
zPop = AkuData(10000)[,1:7]
lc = rbind(LagDiff(8,4),PeriodDiff(8,4))
rownames(lc) = c("diffQ1","diffQ2","diffQ3","diffQ4","diffYearMean")
CalibrateSSBpanel(grossSample=z,calmodel="~ sex*age", partition=c("year","q"),popData=zPop, 
       y=c("unemployed","workforce"),id="id",wave=c("year","q"),
       numerator="unemployed",linComb=lc)
}
\seealso{
\code{\link{CalibrateSSB}}, \code{\link{PanelEstimation}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CalibrateSSB.R
\encoding{UTF8}
\name{CalibrateSSB}
\alias{CalibrateSSB}
\title{Calibration weighting and estimation}
\usage{
CalibrateSSB(
  grossSample,
  calmodel = NULL,
  response = "R",
  popTotals = NULL,
  y = NULL,
  by = NULL,
  partition = NULL,
  lRegmodel = NULL,
  popData = NULL,
  samplingWeights = NULL,
  usePackage = "survey",
  bounds = c(-Inf, Inf),
  calfun = "linear",
  onlyTotals = FALSE,
  onlyw = FALSE,
  uselRegWeights = FALSE,
  ids = NULL,
  residOutput = TRUE,
  leverageOutput = FALSE,
  yOutput = TRUE,
  samplingWeightsOutput = FALSE,
  dropResid2 = TRUE,
  wGrossOutput = TRUE,
  wave = NULL,
  id = NULL,
  extra = NULL,
  allowNApopTotals = NULL,
  partitionPrint = NULL,
  ginvtol = 1e-06,
  boundstol = 1e-06,
  resids_by_lm = FALSE,
  ...
)
}
\arguments{
\item{grossSample}{Input data, typically a data frame, tibble, or data.table.
If \code{grossSample} is not a classic data frame,
it will be coerced to one internally.}

\item{calmodel}{Formula defining the linear structure of the calibration
model.}

\item{response}{Variable name of response indicator (net sample when 1).}

\item{popTotals}{Population totals (similar to population totals as output).
If \code{popTotals} is not a classic data frame,
it will be coerced to one internally.}

\item{y}{Names of variables of interest. Can be a list similar to "by"
below.}

\item{by}{Names of the variables that define the "estimation domains". If
NULL (the default option) or NA estimates refer to the whole population. Use
list for multiple specifications (resulting in list as output).}

\item{partition}{Names of the variables that define the "calibration
domains" for the model. NULL (the default) implies no calibration domains.}

\item{lRegmodel}{Formula defining the linear structure of a logistic
regression model.}

\item{popData}{Population data, typically a data frame, tibble, or data.table.
If \code{popData} is not a classic data frame,
it will be coerced to one internally.}

\item{samplingWeights}{Name of the variable with initial weights for the
sampling units.}

\item{usePackage}{Specifying the package to be used: "survey" (the default),
"ReGenesees" or "none".}

\item{bounds}{Bounds for the calibration weights. When ReGenesees: Allowed
range for the ratios between calibrated and initial weights. The default is
c(-Inf,Inf).}

\item{calfun}{The distance function for the calibration process; the default
is 'linear'.}

\item{onlyTotals}{When TRUE: Only population totals are returned.}

\item{onlyw}{When TRUE: Only the calibrated weights are returned.}

\item{uselRegWeights}{When TRUE: Weighted logistic regression is performed
as a first calibration step.}

\item{ids}{Name of sampling unit identifier variable.}

\item{residOutput}{Residuals in output when TRUE. FALSE is default.}

\item{leverageOutput}{Leverages in output when TRUE. FALSE is default.}

\item{yOutput}{y in output when TRUE. FALSE is default.}

\item{samplingWeightsOutput}{samplingWeights in output when TRUE. FALSE is
default.}

\item{dropResid2}{When TRUE (default) and when no missing population totals
\itemize{
\item only one set of residuals in output.
}}

\item{wGrossOutput}{wGross in output when TRUE (default) and when NA
popTotals.}

\item{wave}{Time or another repeat variable (to be included in output).}

\item{id}{Identifier variable (to be included in output).}

\item{extra}{Variables for the extra dataset (to be included in output).}

\item{allowNApopTotals}{When TRUE missing population totals are allowed.
Results in error when FALSE and warning when NULL.}

\item{partitionPrint}{When TRUE partition progress is printed.
Automatic decision when NULL (about 1 min total computing time).}

\item{ginvtol}{Tolerance parameter for the generalized inverse used when
\code{usePackage = "none"} and also for the computation of \code{resids}
when \code{resids_by_lm = FALSE} and \code{leverageOutput = FALSE}. The default
(\code{1e-06}) is chosen with robustness of the function in mind, beyond
machine precision effects. A more standard value is \code{1.5e-08},
similar to \code{ginv()} in the MASS package.}

\item{boundstol}{Tolerance parameter used in iterations involving bounds
when \code{usePackage = "none"}.}

\item{resids_by_lm}{When \code{FALSE} (default) and \code{leverageOutput = FALSE},
the computation of \code{resids} is based on a generalized inverse.
Otherwise, the computations go via the \code{lm()} function.}

\item{...}{Further arguments sent to underlying functions.}
}
\value{
Unless onlyTotals or onlyw is TRUE, the output is an object of class calSSB. That is, a list with
elements: \item{popTotals}{Population totals.} \item{w}{The calibrated
weights.} \item{wGross}{Calibrated gross sample weights when NA popTotals.}
\item{estTM}{Estimates (with standard error).} \item{resids}{Residuals,
reduced model when NA popTotals.} \item{resids2}{Residuals, full model.}
\item{leverages}{Diagonal elements of hat-matrix, reduced model when NA
popTotals.} \item{leverages2}{Diagonal elements of hat-matrix, full model.}
\item{y}{as input} \item{samplingWeights}{as input}
\item{wave}{as input or via CrossStrata}
\item{id}{as input}
\item{extra}{as input}
}
\description{
Compute weights by calibration and corresponding estimates, totals and
residuals
}
\details{
When popTotals as input is NULL, population totals are computed from popData
(when available) or from grossSample. Some elements of popTotals may be
missing (not allowed when using ReGenesees). When using "ReGenesees", both
weiging and estimation are done by that package. When using "survey", only
calibration weiging are done by that package.
The parameters \code{wave}, \code{id} and \code{extra} have no effect on the
computations, but result in extra elements in output
(to be used by WideFromCalibrate() later).
}
\examples{

# Generates data  - two years
z    <- AkuData(3000)  # 3000 in each quarter
zPop <- AkuData(10000)[,1:7]

# Calibration using "survey"
a <- CalibrateSSB(z, calmodel = "~ sex*age",
                 partition = c("year","q"),  # calibrate within quarter
                 popData = zPop, y = c("unemployed","workforce"),
                 by = c("year","q")) # Estimate within quarter
head(a$w) # calibrated weights
a$estTM   # estimates
a$popTotals   # popTotals used as input below


# Calibration, no package, popTotals as input
b <- CalibrateSSB(z, popTotals=a$popTotals, calmodel="~ sex*age",
      partition = c("year","q"), usePackage = "none", y = c("unemployed","workforce"))
max(abs(a$w-b$w)) # Same weights as above

print(a)
print(b)

\dontrun{
require(ReGenesees)
# Calibration and estimation via ReGenesees
CalibrateSSB(z, calmodel = "~ sex*age",
             partition = c("year","q"),  # calibrate within quarter
             popData = zPop, usePackage = "ReGenesees",
             y = c("unemployed","workforce"),
             by = c("year","q")) # Estimate within quarter
}

}
\seealso{
\code{\link{CalSSBobj}}, \code{\link{WideFromCalibrate}},  \code{\link{PanelEstimation}}, \code{\link{CalibrateSSBpanel}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AkuData.R
\name{AkuData}
\alias{AkuData}
\title{Generate test data}
\usage{
AkuData(n)
}
\arguments{
\item{n}{Number of observations within each quarter.}
}
\value{
A data frame with the following variables: \item{id}{Sample unit
identifier} \item{year}{Year} \item{q}{Quarter} \item{month}{Month}
\item{R}{Response indicator} \item{age}{Age group} \item{sex}{Education
group} \item{famid}{Family identifier} \item{unemployed}{Unemployed}
\item{workforce}{In workforce}
}
\description{
Generate test data of eight quarters
}
\examples{

# Generates data - two years
z = AkuData(3000) # 3000 in each quarter

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CalibrateSSB-package.R
\docType{package}
\encoding{UTF8}
\name{CalibrateSSB-package}
\alias{CalibrateSSB-package}
\title{Weighting and Estimation for Panel Data with Non-Response}
\description{
CalibrateSSB is an R-package that handles repeated surveys with partially
overlapping samples. Initially the samples are weighted by linear
calibration using known or estimated population totals. A robust model based
covariance matrix for all relevant estimated totals is calculated from the
residuals according to the calibration model. Alternatively a design based
covariance matrix is calculated in a very similar way. A cluster robust
version is also possible. In the case of estimated populations totals the
covariance matrix is adjusted by utilizing the theory of Särndal and
Lundström (2005). Variances of linear combinations (changes and averages)
and ratios are calculated from this covariance matrix. The linear
combinations and ratios can involve variables within and/or between sample
waves. \cr
}
\references{
Langsrud, Ø (2016): \dQuote{A variance estimation R-package for
repeated surveys - useful for estimates of changes in quarterly and annual
averages}, \emph{Romanian Statistical Review} nr. 2 / 2016, pp. 17-28.
CONFERENCE: \emph{New Challenges for Statistical Software - The Use of R in
Official Statistics}, Bucharest, Romania, 7-8 April. \cr

Särndal, C.-E. and Lundström, S. (2005): \emph{Estimation in Surveys with
Nonresponse}, John Wiley and Sons, New York.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/statisticsnorway/ssb-calibratessb}
  \item \url{https://statisticsnorway.github.io/ssb-calibratessb/}
  \item Report bugs at \url{https://github.com/statisticsnorway/ssb-calibratessb/issues}
}

}
\author{
\strong{Maintainer}: Øyvind Langsrud \email{oyl@ssb.no} (\href{https://orcid.org/0000-0002-1380-4396}{ORCID})

Authors:
\itemize{
  \item Øyvind Langsrud \email{oyl@ssb.no} (\href{https://orcid.org/0000-0002-1380-4396}{ORCID})
}

}
\keyword{calibration}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calSSB.R
\name{print.calSSB}
\alias{print.calSSB}
\title{Print method for calSSB}
\usage{
\method{print}{calSSB}(x, digits = max(getOption("digits") - 3, 3), ...)
}
\arguments{
\item{x}{calSSB object}

\item{digits}{positive integer.  Minimum number of significant digits to be used for printing most numbers.}

\item{\dots}{further arguments sent to the underlying}
}
\value{
Invisibly returns the original object.
}
\description{
Print method for calSSB
}
\keyword{print}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PanelEstimation.R
\name{PanelEstimation}
\alias{PanelEstimation}
\title{Variance estimation for panel data}
\usage{
PanelEstimation(
  x,
  numerator,
  denominator = NULL,
  linComb = matrix(0, 0, n),
  linComb0 = NULL,
  estType = "robustModel",
  leveragePower = 1/2,
  group = NULL,
  returnCov = FALSE,
  usewGross = TRUE
)
}
\arguments{
\item{x}{Output from wideFromCalibrate.}

\item{numerator}{y variable name or number.}

\item{denominator}{y variable name or number.}

\item{linComb}{Matrix defining linear combinations of waves.}

\item{linComb0}{Linear combination matrix to be used prior to ratio
calculations.}

\item{estType}{Estimation type: "robustModel" (default), "ssbAKU",
"robustModelww", "robustModelGroup" or "robustModelGroupww" (see below)}

\item{leveragePower}{Power used when adjusting residuals using leverages.}

\item{group}{Extra variable name or number for cluster robust estimation.}

\item{returnCov}{Return covariance matrices instead of variance vectors.}

\item{usewGross}{Use wGross (if avaliable) instead of design weights to
adjust covariance matrix in the case of NA popTotals}
}
\value{
\item{wTot}{Sum of weights} \item{estimates}{Ordinary estimates}
\item{linCombs}{Estimates of linear combinations}
\item{varEstimates}{Variance of estimates} \item{varLinCombs}{Variance of
estimates of linear combinations} When denominator is specified the above
output refer to ratios. Then, similar output for numerator and denominator
are also included.
}
\description{
Variance estimation of linear combinations of totals and ratios based on
output from wideFromCalibrate
}
\details{
When denominator=NULL, only estimates for a single y-variable (numerator)
are calculated. When denominator is specified, estimates for numerator,
denominator and ratio are calculated. The default estimation type parameter,
"robustModel", is equation (12) in paper. "ssbAKU" is (16), "robustModelww"
is (9) and "robustModelGroup" and "robustModelGroupww" are cluster robust
variants based on \eqn{(w-1)^2} and \eqn{w^2} .
}
\examples{

# Generates data  - two years
z    = AkuData(3000)  # 3000 in each quarter
zPop = AkuData(10000)[,1:7]

# Calibration and "WideFromCalibrate"
b = CalibrateSSB(z,calmodel="~ sex*age", partition=c("year","q"),
        popData=zPop, y=c("unemployed","workforce"))
bWide = WideFromCalibrate(b,CrossStrata(z[,c("year","q")]),z$id)

# Define linear combination matrix
lc = rbind(LagDiff(8,4),PeriodDiff(8,4))
rownames(lc) = c("diffQ1","diffQ2","diffQ3","diffQ4","diffYearMean")
colnames(lc) = colnames(head(bWide$y[[1]]))
lc

# Unemployed: Totals and linear combinations
d1=PanelEstimation(bWide,"unemployed",linComb=lc)  #

# Table of output
cbind(tot=d1$estimates,se=sqrt(d1$varEstimates))
cbind(tot=d1$linCombs,se=sqrt(d1$varLinCombs))

# Ratio: Totals and linear combinations
d=PanelEstimation(bWide,numerator="unemployed",denominator="workforce",linComb=lc)
cbind(tot=d$estimates,se=sqrt(d$varEstimates))
cbind(tot=d$linCombs,se=sqrt(d$varLinCombs))

\dontrun{
# Calibration when som population totals unknown (edu)
# Leverages in output (will be used to adjust residuals)
# Cluster robust estimation (families/famid)
b2 = CalibrateSSB(z,popData=zPop,calmodel="~ edu*sex + sex*age",
           partition=c("year","q"), y=c("unemployed","workforce"),
           leverageOutput=TRUE)
b2Wide = WideFromCalibrate(b2,CrossStrata(z[,c("year","q")]),z$id,extra=z$famid)
d2 = PanelEstimation(b2Wide,"unemployed",linComb=lc,group=1,estType = "robustModelGroup")
cbind(tot=d2$linCombs,se=sqrt(d2$varLinCombs))
}


# Yearly mean before ratio calculation (linComb0)
# and difference between years (linComb)
g=PanelEstimation(bWide,numerator="unemployed",denominator="workforce",
    linComb= LagDiff(2),linComb0=Period(8,4))
cbind(tot=g$linCombs,se=sqrt(g$varLinCombs))

}
\seealso{
\code{\link{CalibrateSSB}}, \code{\link{CalSSBobj}}, \code{\link{WideFromCalibrate}}, \code{\link{CalibrateSSBpanel}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setAndGetTotal.R
\name{OrderedVarNames}
\alias{OrderedVarNames}
\title{OrderedVarNames}
\usage{
OrderedVarNames(x, sep = ":")
}
\arguments{
\item{x}{input}

\item{sep}{character string used to separate}
}
\value{
output
}
\description{
OrderedVarNames
}
\examples{
z <- data.frame(A = factor(c("a", "b", "c")), B = factor(1:2), C = 1:6)
x <- colnames(model.matrix(~B * C * A, z))
OrderedVarNames(x)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PanelEstimation.R
\encoding{UTF8}
\name{LinCombMatrix}
\alias{LinCombMatrix}
\alias{PeriodDiff}
\alias{Period}
\alias{LagDiff}
\title{Creation of linear combination matrices}
\usage{
LinCombMatrix(
  n,
  period = NULL,
  lag = NULL,
  k = 0,
  takeMean = TRUE,
  removerows = TRUE,
  overlap = FALSE
)

LagDiff(n, lag = 1, removerows = TRUE)

Period(
  n,
  period = 1,
  k = 0,
  takeMean = TRUE,
  removerows = TRUE,
  overlap = FALSE
)

PeriodDiff(
  n,
  period = 1,
  lag = period,
  k = 0,
  takeMean = TRUE,
  removerows = TRUE,
  overlap = FALSE
)
}
\arguments{
\item{n}{Number of variables}

\item{period}{Number of variables involved in each period}

\item{lag}{Lag used for difference calculation}

\item{k}{Shift the start of each period}

\item{takeMean}{Calculate mean over each period (sum when FALSE)}

\item{removerows}{Revove incomplete rows}

\item{overlap}{Overlap between periods (moving averages)}
}
\value{
Linear combination matrix
}
\description{
Create matrices for changes (LagDiff), means (Period) and mean changes
(PeriodDiff).
}
\note{
It can be useful to add row names to the resulting matrix before
further use.
}
\examples{

# We assume two years of four quarters (n=8)

# Quarter to quarter differences
LagDiff(8)

# Changes from same quarter last year
LagDiff(8,4)

# Yearly averages
Period(8,4)

# Moving yearly averages
Period(8,4,overlap=TRUE)

# Difference between yearly averages
PeriodDiff(8,4) # Also try n=16 with overlap=TRUE/FALSE

# Combine two variants and add row names
lc = rbind(LagDiff(8,4),PeriodDiff(8,4))
rownames(lc) = c("diffQ1","diffQ2","diffQ3","diffQ4","diffYearMean")
lc

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calSSB.R
\name{CalSSBobj}
\alias{CalSSBobj}
\title{Create or modify a CalSSB object}
\usage{
CalSSBobj(
  x = NULL,
  y = NULL,
  w = NULL,
  wGross = NULL,
  resids = NULL,
  resids2 = NULL,
  leverages = NULL,
  leverages2 = NULL,
  samplingWeights = NULL,
  extra = NULL,
  id = NULL,
  wave = NULL
)
}
\arguments{
\item{x}{NULL or an  existing calSSB object}

\item{y}{y}

\item{w}{w}

\item{wGross}{wGross}

\item{resids}{resids}

\item{resids2}{resids2}

\item{leverages}{leverages}

\item{leverages2}{leverages2}

\item{samplingWeights}{samplingWeights}

\item{extra}{extra}

\item{id}{id}

\item{wave}{wave}
}
\value{
A CalSSB object. That is, an object of the type retuned by \code{\link{CalibrateSSB}}.
}
\description{
The elements of the CalSSB object are taken directly from the input parameters.
}
\note{
If x is a ReGenesees/cal.analytic object, this function is a wrapper to \code{\link{CalSSBobjReGenesees}}.
}
\examples{
#' # Generates data - two years
z <- AkuData(3000)  # 3000 in each quarter
zPop <- AkuData(10000)[, 1:7]

# Create a CalSSB object by CalibrateSSB
b <- CalibrateSSB(z, calmodel = "~ sex*age", partition = c("year", "q"), popData = zPop, 
                  y = c("unemployed", "workforce"))

# Modify the CalSSB object
a <- CalSSBobj(b, w = 10*b$w, wave = CrossStrata(z[, c("year", "q")]), id = z$id)

# Use the CalSSB object as input ...
PanelEstimation(WideFromCalibrate(a), "unemployed", linComb = PeriodDiff(8, 4))

# Create CalSSB object without x as input
CalSSBobj(y = b$y, w = 10*b$w, resids = b$resids, wave = CrossStrata(z[, c("year", "q")]), 
          id = z$id)

}
\seealso{
\code{\link{CalibrateSSB}}, \code{\link{CalSSBobjReGenesees}}, \code{\link{WideFromCalibrate}},  \code{\link{PanelEstimation}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calSSB.R
\name{CalSSBobjReGenesees}
\alias{CalSSBobjReGenesees}
\title{Create a CalSSB object from a ReGenesees/cal.analytic object}
\usage{
CalSSBobjReGenesees(
  x,
  y,
  samplingWeights = NULL,
  extra = NULL,
  id = NULL,
  wave = NULL
)
}
\arguments{
\item{x}{Output from ReGenesees::e.calibrate() (object of class cal.analytic)}

\item{y}{formula or variable names}

\item{samplingWeights}{NULL, TRUE (capture from x), formula, variable name or vector of data}

\item{extra}{NULL, formula, variable names or matrix of data}

\item{id}{NULL, TRUE (ids from x), formula, variable name or vector of data}

\item{wave}{NULL,  formula, variable name or vector of data}
}
\value{
A CalSSB object. That is, an object of the type retuned by \code{\link{CalibrateSSB}}.
}
\description{
Create a CalSSB object from a ReGenesees/cal.analytic object
}
\examples{
\dontrun{
# Generates data - two years
z <- AkuData(3000)  # 3000 in each quarter
zPop <- AkuData(10000)[, 1:7]
z$samplingWeights <- 1
z$ids <- 1:NROW(z)

# Create a ReGenesees/cal.analytic object
library("ReGenesees")
desReGenesees <- e.svydesign(z[z$R == 1, ], ids = ~ids, weights = ~samplingWeights)
popTemplate <- pop.template(data = desReGenesees, calmodel = ~sex * age, partition = ~year + q)
popTotals <- fill.template(universe = zPop, template = popTemplate)
calReGenesees <- e.calibrate(design = desReGenesees, df.population = popTotals)

# Create CalSSB objects from a ReGenesees/cal.analytic object
CalSSBobjReGenesees(calReGenesees, y = ~unemployed + workforce, id = TRUE, 
                    samplingWeights = TRUE, extra = ~famid)
a <- CalSSBobjReGenesees(calReGenesees, y = c("unemployed", "workforce"), 
                         id = "id", extra = "famid", wave = c("year", "q"))

# Use the CalSSB object as input ...
PanelEstimation(WideFromCalibrate(a), "unemployed", linComb = PeriodDiff(8, 4))

}
}
\seealso{
\code{\link{CalibrateSSB}}, \code{\link{CalSSBobj}}, \code{\link{WideFromCalibrate}},  \code{\link{PanelEstimation}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AkuData.R
\docType{data}
\name{testDataBasis}
\alias{testDataBasis}
\title{testDataBasis}
\description{
Data used by \code{\link{AkuData}}
}
\keyword{datasets}
\keyword{internal}
